'From Squeak2.7 of 5 January 2000 [latest update: #1762] on 19 January 2000 at 11:24:36 am'!
Object subclass: #TextFormatter
	instanceVariableNames: 'match formattingArray '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swiki'!
!TextFormatter commentStamp: '<historical>' prior: 0!
TextFormatter

This is a general text formatter. You create an instance which is a text formatter. Then, using 'format:' you format a text to get a formatted stream back.

There are two type of formatting that can be added: mapping and action. A mapping is when an original in the text gets replaced with the replacement in the formatted stream. An action is when the text between the two elements is send into the code block and the result is sent to the formatted stream. In case of conflict, the longer match wins; for instance, in a SwikiFormatter, '----' might match to '<HR>', while '-' might match to '<LI>'.!


!TextFormatter methodsFor: 'accessing'!
addAction: block from: beginning to: end
	"When the Action is performed, the text is scanned for things that start with beginning
	and stop with end. Whatever is between these two is then put into the block and what
	gets returned is fed to the formatted document."
	"The format for an Action is a 3 element array of (beginning end block)."
	| starts formatter |

	starts _ beginning at: 1.
	match add: starts.
	formatter _ Array new: 3.
	beginning size = 1
		ifTrue: [formatter at: 1 put: '']
		ifFalse: [formatter at: 1 put: (beginning copyFrom: 2 to: beginning size)].
	formatter at: 2 put: end.
	formatter at: 3 put: block.
	self addFormatter: formatter at: (starts asInteger).! !

!TextFormatter methodsFor: 'accessing'!
addMappingFrom: original to: replacement
	"A Mapping is when the text is scanned and all instances of original are mapped to
	their replacements."
	"The format for a Mapping is an 3 element array of (original nil replacement)."
	| starts formatter |

	starts _ original at: 1.
	match add: starts.
	formatter _ Array new: 3.
	original size = 1
		ifTrue: [formatter at: 1 put: '']
		ifFalse: [formatter at: 1 put: (original copyFrom: 2 to: original size)].
	formatter at: 3 put: replacement.
	self addFormatter: formatter at: (starts asInteger).! !

!TextFormatter methodsFor: 'accessing'!
remove: key
	"Removes a formatter (action, mapping) from the TextFormatter."
	"It remove it if it matches the orginal for a mapping or the beginning for an action."
	| starts formatters ends |

	starts _ key at: 1. "First character in the key."
	formatters _ formattingArray at: (starts asInteger).
	formatters ifNil: ["There is no matching formatter, so stop" ^nil]
		ifNotNil: [((key size) = 1) ifTrue: [ends _ ''] ifFalse: [ends _ key copyFrom: 2 to: (key size)].
			formatters do: [:element | ((element at: 1) = ends) ifTrue: [formatters remove: element]].
			((formatters size) = 0) ifTrue: [
				formattingArray at: (starts asInteger) put: nil. "Removes formatter"
				match remove: starts. "Removes from match"
			]
		].
	^nil! !


!TextFormatter methodsFor: 'initializing'!
initialize
	match _ CharacterSet new.
	formattingArray _ Array new: 255.! !


!TextFormatter methodsFor: 'private'!
action: action value: value text: text
	^(action copy fixTemps) value: text! !

!TextFormatter methodsFor: 'private'!
addFormatter: formatter at: position
	"Adds a formatter to the position in the formattingArray."
	| formatters |

	formatters _ formattingArray at: position.
	formatters isNil
		ifTrue: [formatters _ SortedCollection new sortBlock: [:x :y | (x at: 1) size >= (y at: 1) size].
			formattingArray at: position put: formatters].
	formatters add: formatter.
! !

!TextFormatter methodsFor: 'private'!
format: text actionValue: value onTo: stream
	"Apply all the formatting on text and put it onto stream."
	| pos i maxSize formatters index formatter |

	"i is the position of a match"
	pos _ 1. "Position in text, marking the seperation of processed and yet to be processed"
	[(i _ text indexOfAnyOf: match startingAt: pos) > 0]
		whileTrue: [stream nextPutAll: (text copyFrom: pos to: i-1). "No formatting in this space"
			maxSize _ text size - i.
			formatters _ formattingArray at: (text at: i) asInteger.
			index _ 1.
			[formatters size >= index] "Index of Formatters"
				whileTrue: [ "see if formatters match"
					formatter _ formatters at: index.
					"Check that size of formatter is allowed."
					(maxSize >= (formatter at: 1) size) ifTrue: [
						"Check if the formatter matches"
						((text copyFrom: (i + 1) to: (i + (formatter at: 1) size)) = (formatter at: 1))
							ifTrue: [ index _ formatters size + 1. "Match => exit loop"
								"Set i to beyond beginning marker"
								pos _ i + 1 + (formatter at: 1) size.
								(formatter at: 2)
									ifNil: [ "Mapping"
											stream nextPutAll: (formatter at: 3)
										]
									ifNotNil: [ "Action"
											"This does not allow nesting, but that's probably good."
											i _ text findString: (formatter at: 2) startingAt: pos.
											"Check for no closing"
											i > 0 ifTrue: [stream nextPutAll: (self action: (formatter at: 3) value: value text: (text copyFrom: pos to: i-1)).
													pos _ i + (formatter at: 2) size
												]
										]
								]
						].
					index _ index + 1.
				].
				"Check for no formatter matched"
				((formatters size) = (index - 1)) ifTrue: [stream nextPut: (text at: i). pos _ i + 1]
		].
		"Add last section if there is one."
		(pos <= text size) ifTrue: [stream nextPutAll: (text copyFrom: pos to: (text size))].
	 ^stream! !


!TextFormatter methodsFor: 'processing'!
format: text
	"Apply all the formatting to the text and return it as a stream"
	| return |

	"Create a new stream to write it on."
	return _ WriteStream on: (String new: text size).
	self format: text onTo: return.
	^return contents
! !

!TextFormatter methodsFor: 'processing'!
format: text onTo: stream
	^self format: text actionValue: nil onTo: stream! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

TextFormatter class
	instanceVariableNames: ''!

!TextFormatter class methodsFor: 'instance creation'!
fixForIEFormatter
	"Creates an instance that fixes the mark-ups for Internet Explorer.
	These are simply the alternatives for '&', '<', and '>'."
	| fm |

	fm _ self new.
	fm initialize.
	fm addMappingFrom: '<' to: '&lt;'.
	fm addMappingFrom: '>' to: '&gt;'.
	fm addMappingFrom: '&' to: '&amp;'.
	^fm! !
