'From Squeak2.8alpha of 19 January 2000 [latest update: #1974] on 30 March 2000 at 8:39:28 am'!
SimpleButtonMorph subclass: #GradientButtonMorph
	instanceVariableNames: 'topColor bottomColor reverseFillStyle saveFill '
	classVariableNames: 'DelayTime '
	poolDictionaries: ''
	category: 'Morphic Documentation example'!
Morph subclass: #WorkspacePane
	instanceVariableNames: 'contents bindings '
	classVariableNames: 'BackgroundForm '
	poolDictionaries: ''
	category: 'Morphic Documentation example'!

!GradientButtonMorph commentStamp: 'jlb 3/27/2000 09:46' prior: 0!
I represent a SimpleButtonMorph which appears on the screen as a gradient filled button. This gradient gives the appearance of a "3D" look.

When the button is released, it will flash 3 times. Class variable helps determine the length of the flash time.

Instance variables topColor and bottomColor determine the top and bottom colors of the gradient. reverseFillStyle caches a copy of the gradient going in the reverse direction, for feedback when the button is depressed. saveFill is used during the release cycle.!

!GradientButtonMorph reorganize!
('color' cacheFillStyle gradientColor: gradientColor:bottomColor: labelColor:)
('event handling' mouseDown: mouseMove: mouseUp:)
('geometry' layoutChanged position:)
('initialize' initialize label: setDefaultLabel)
!


!GradientButtonMorph methodsFor: 'color' stamp: 'jlb 3/26/2000 09:34'!
cacheFillStyle
	| fill |
	"Make receiver use a solid fill style (e.g., a simple color)"
	fill _ GradientFillStyle ramp: {0.0 -> topColor. 1.0 -> bottomColor}.
	fill	origin: self topLeft ;
		direction: 0 @ self bounds extent y ;
		normal: self bounds extent x @ 0 ;
		radial: false.
	self fillStyle: fill.

	" reverse ramps the colors in the opposite direction "
	reverseFillStyle _ fill copy.
	reverseFillStyle colorRamp: {0.0 -> bottomColor. 1.0 -> topColor}.

! !

!GradientButtonMorph methodsFor: 'color' stamp: 'jlb 3/24/2000 15:38'!
gradientColor: theColor 

	topColor _ theColor.
	bottomColor _  (theColor mixed: 0.5 with: Color black).
	self cacheFillStyle.
	self changed.
! !

!GradientButtonMorph methodsFor: 'color' stamp: 'jlb 3/24/2000 15:38'!
gradientColor: color1 bottomColor: color2

	topColor _ color1.
	bottomColor _ color2.
	self cacheFillStyle.
	self changed.! !

!GradientButtonMorph methodsFor: 'color' stamp: 'jlb 3/24/2000 15:38'!
labelColor: theColor
	| sm |
	(sm _ self findA: StringMorph)
		ifNotNil: [sm color: theColor.].
! !

!GradientButtonMorph methodsFor: 'event handling' stamp: 'jlb 3/24/2000 17:29'!
mouseDown: evt
	self cacheFillStyle.
	saveFill _ self fillStyle.
	self fillStyle: reverseFillStyle.
	self changed.
	self refreshWorld.

! !

!GradientButtonMorph methodsFor: 'event handling' stamp: 'jlb 3/24/2000 16:26'!
mouseMove: evt 
	actWhen == #buttonDown ifTrue: [^ self].
	(self containsPoint: evt cursorPoint)
		ifTrue: 
			[self fillStyle = reverseFillStyle
				ifFalse: 
					[self fillStyle: reverseFillStyle.
					self changed.
					self world displayWorld].
			(actWhen == #whilePressed and: [evt anyButtonPressed])
				ifTrue: 
					[self doButtonAction.
					evt hand noteSignificantEvent: evt]]
		ifFalse: [self fillStyle = saveFill
				ifFalse: 
					[self fillStyle: saveFill.
					self changed.
					self world displayWorld]]! !

!GradientButtonMorph methodsFor: 'event handling' stamp: 'jlb 3/29/2000 21:42'!
mouseUp: evt 
	" flash the button three times when the button is released"
	| delay time elapsed |
	(self containsPoint: evt cursorPoint)
		ifTrue: [1 to: 3 do: 
				[:i | 
				time _ Time millisecondClockValue.
				self fillStyle: saveFill.
				self changed.
				self refreshWorld.
				elapsed _ Time millisecondClockValue - time.
				elapsed > 0
					ifTrue: 
						[delay _ Delay forMilliseconds: (elapsed min: DelayTime).
						delay wait].
				self fillStyle: reverseFillStyle.
				self changed.
				self refreshWorld.
				elapsed > 0 ifTrue: [delay wait]]].
	self fillStyle: saveFill.
	self changed.
	self refreshWorld.
	(actWhen == #buttonUp and: [self containsPoint: evt cursorPoint])
		ifTrue: [self doButtonAction]! !

!GradientButtonMorph methodsFor: 'geometry' stamp: 'jlb 3/24/2000 16:28'!
layoutChanged
	| sm |

	super layoutChanged.
	( sm _ self findA: StringMorph )
		ifNotNil: [ 	sm position: self center - (sm extent // 2). ].

	self cacheFillStyle.
! !

!GradientButtonMorph methodsFor: 'geometry' stamp: 'jlb 3/24/2000 16:59'!
position: thePoint	
	super position: thePoint.
	self cacheFillStyle.
! !

!GradientButtonMorph methodsFor: 'initialize' stamp: 'jlb 3/24/2000 18:05'!
initialize

	super initialize.
	self useSquareCorners.
	bounds _ 0 @ 0 extent: 96 @ ( self class buttonHeight) .
	self setDefaultLabel.
	self borderColor: #raised.
	self borderWidth: 1.
	self useGradientFill.
	topColor _ Color lightGray.
	bottomColor _ Color black.
	self cacheFillStyle.


! !

!GradientButtonMorph methodsFor: 'initialize' stamp: 'jlb 3/24/2000 13:41'!
label: aString

	| oldLabel m |
	(oldLabel _ self findA: StringMorph)
		ifNotNil: [oldLabel delete].
	m _ StringMorph contents: aString font: TextStyle defaultFont.
	m position: self center - (m extent // 2).
	self addMorph: m.
	m lock! !

!GradientButtonMorph methodsFor: 'initialize' stamp: 'jlb 3/26/2000 09:50'!
setDefaultLabel
	self label: 'Untitled'.
! !


!GradientButtonMorph class reorganize!
('class initialization' initialize)
('constants' buttonHeight)
!


!GradientButtonMorph class methodsFor: 'class initialization' stamp: 'jlb 3/24/2000 17:47'!
initialize
	DelayTime _ 60.! !

!GradientButtonMorph class methodsFor: 'constants' stamp: 'jlb 3/24/2000 13:47'!
buttonHeight
	^ 22! !


!PluggableTextMorph methodsFor: 'initialization' stamp: 'jlb 3/25/2000 09:38'!
editColor: theColor
	textMorph color: theColor.! !

!PluggableTextMorph methodsFor: 'private' stamp: 'jlb 3/26/2000 22:51'!
textMorph
	^ textMorph! !


!SystemWindow methodsFor: 'geometry' stamp: 'jlb 3/27/2000 09:12'!
minimumExtent
	| ext |
	"This returns the minimum extent that the window may be resized to"
	(ext _ self valueOfProperty: #minimumExtent)
		ifNotNil:
			[^ ext].
	^ 100 @ 80! !

!SystemWindow methodsFor: 'geometry' stamp: 'jlb 3/27/2000 09:13'!
minimumExtent: aPoint
	"This sets the minimum extent to which the window may be resized to"
	self setProperty: #minimumExtent toValue: aPoint.
! !


!WorkspacePane commentStamp: 'jlb 3/27/2000 09:43' prior: 0!
I represent a modified WorkspacePane.

I place a row of 5 buttons across the top of the window, which allows quick and easy access to several window opening functions. I can open a Browser, a Workspace, a Transcript, a ChangeSorter and a FileList from this button row.

Class variable BackgroundForm is the default form to use for my background.

Instance variable content and bindings are conterparts to Workspace --

This is part of Jim Bensons essay on Morphic series.!

!WorkspacePane reorganize!
('accessing' contents contents:)
('backgrounds' setBackground: setDefaultBackground)
('bindings' bindingOf: setBindings:)
('color' color: editTextColor setEditTextColor setEditTextColor: textColor:)
('geometry' extent:)
('initialize' addButtons initialExtent initialize)
('menus' addModelItemsToWindowMenu: codePaneMenu:shifted: saveContentsInFile)
('private' privateMoveBy:)
('workspace stuff' acceptContents: defaultContents selectedMessageName)
!


!WorkspacePane methodsFor: 'accessing' stamp: 'jlb 3/24/2000 13:53'!
contents
	"Answer the contents that the receiver is holding--presumably a string."

	^contents! !

!WorkspacePane methodsFor: 'accessing' stamp: 'jlb 3/24/2000 13:53'!
contents: textOrString 
	"Set textOrString to be the contents of the receiver."

	contents _ textOrString "asString"! !

!WorkspacePane methodsFor: 'backgrounds' stamp: 'jlb 3/28/2000 20:24'!
setBackground: theForm 
	" Assign the background to the PluggableTextMorph"
	| fill tm |
	(tm _ self findA: PluggableTextMorph)
		ifNotNil: 
			[theForm
				ifNil: 
					[tm removeProperty: #fillStyle.
					tm color: Color transparent.
					^ self].
			fill _ BitmapFillStyle fromForm: theForm.
			fill origin: 1@1.
			tm fillStyle: fill]! !

!WorkspacePane methodsFor: 'backgrounds' stamp: 'jlb 3/26/2000 22:26'!
setDefaultBackground
	BackgroundForm ifNotNil: [
	self setBackground: BackgroundForm ].! !

!WorkspacePane methodsFor: 'bindings' stamp: 'jlb 3/25/2000 19:17'!
bindingOf: aString
	bindings isNil
		ifTrue: [bindings _ Dictionary new].
	(bindings includesKey: aString)
		ifFalse: [bindings at: aString put: nil].
	^bindings associationAt: aString! !

!WorkspacePane methodsFor: 'bindings' stamp: 'jlb 3/25/2000 19:17'!
setBindings: aDictionary
	"Sets the Workspace to use the specified dictionary as its namespace"

	bindings _ aDictionary.
! !

!WorkspacePane methodsFor: 'color' stamp: 'jlb 3/24/2000 17:56'!
color: theColor
	| am |
	super color: theColor.
	(am _ self findA: AlignmentMorph)
		ifNotNil: [am color: (theColor darker)].
	! !

!WorkspacePane methodsFor: 'color' stamp: 'jlb 3/26/2000 22:45'!
editTextColor
	" set the text in the edit pane to be theColor"
	| tm |
	(tm _ self findA: PluggableTextMorph)
		ifNil: [ ^ Color black ]
	    ifNotNil: [tm textMorph color].! !

!WorkspacePane methodsFor: 'color' stamp: 'jlb 3/26/2000 22:42'!
setEditTextColor
	ColorPickerMorph new
		sourceHand: self activeHand ;
		target: self ;
		selector: #setEditTextColor: ;
		originalColor: self editTextColor ;
		addToWorld: self world
			near: self fullBounds.! !

!WorkspacePane methodsFor: 'color' stamp: 'jlb 3/26/2000 22:47'!
setEditTextColor: theColor
	self textColor: theColor.! !

!WorkspacePane methodsFor: 'color' stamp: 'jlb 3/26/2000 22:45'!
textColor: theColor 
	" set the text in the edit pane to be theColor"
	| tm |
	(tm _ self findA: PluggableTextMorph) ifNotNil: [tm editColor: theColor]! !

!WorkspacePane methodsFor: 'geometry' stamp: 'jlb 3/29/2000 21:43'!
extent: thePoint 
	| ib am tm buttonHeight tExtent |
	super extent: thePoint.
	ib _ self innerBounds.
	buttonHeight _ GradientButtonMorph buttonHeight.
	(am _ self findA: AlignmentMorph) ifNotNil: [am bounds: (ib topLeft extent: ib width @ buttonHeight)].
	(tm _ self findA: PluggableTextMorph)
		ifNotNil: 
			[tExtent _ ib extent - (0 @ buttonHeight).
			tm bounds extent = tExtent ifFalse: [tm extent: tExtent]]! !

!WorkspacePane methodsFor: 'initialize' stamp: 'jlb 3/28/2000 21:39'!
addButtons
	| am button |
	am _ AlignmentMorph new.
	am borderWidth: 1.
	am color: #raised.
	am inset: 0.
	button _ GradientButtonMorph new label: 'Browser';
			 target: World activeHand;
			 actionSelector: #openBrowser;
			 actWhen: #buttonUp;
			 gradientColor: (Preferences windowColorFor: #Browser);
			 setBalloonText: 'Open up a System Browser window'.
	am addMorph: button.
	button _ GradientButtonMorph new label: 'Workspace';
			 target: World activeHand;
			 actionSelector: #openWorkspace;
			 actWhen: #buttonUp;
			 gradientColor: (Preferences windowColorFor: #StringHolder);
			 setBalloonText: 'Open a Workspace window'.
	am addMorphBack: button.
	button _ GradientButtonMorph new label: 'Transcript';
			 target: World activeHand;
			 actionSelector: #openTranscript;
			 actWhen: #buttonUp;
			 gradientColor: (Preferences windowColorFor: #TranscriptStream);
			 setBalloonText: 'Open a Transcript window'.
	am addMorphBack: button.
	button _ GradientButtonMorph new label: 'Change Sorter';
			 target: World activeHand;
			 actionSelector: #openChangeSorter:;
			 arguments: #(1 );
			 actWhen: #buttonUp;
			 gradientColor: (Preferences windowColorFor: #ChangeSorter);
			 setBalloonText: 'Open a Simple Change Sorter window'.
	am addMorphBack: button.
	button _ GradientButtonMorph new label: 'File List';
			 target: World activeHand;
			 actionSelector: #openFileList;
			 actWhen: #buttonUp;
			 gradientColor: (Preferences windowColorFor: #FileList);
			 setBalloonText: 'Open a File List window'.
	am addMorphBack: button.
	self addMorph: am.
	am color: color! !

!WorkspacePane methodsFor: 'initialize' stamp: 'jlb 3/27/2000 21:23'!
initialExtent
	^ 482 @ 400.! !

!WorkspacePane methodsFor: 'initialize' stamp: 'jlb 3/28/2000 20:22'!
initialize
	| tm |
	super initialize.
	self color: (Preferences windowColorFor: #StringHolder).
	"Initialize the state of the receiver with its default contents."
	contents _ self defaultContents.
	self addButtons. 

	tm _ PluggableTextMorph
				on: self
				text: #contents
				accept: #acceptContents:
				readSelection: nil
				menu: #codePaneMenu:shifted:.
	tm position: 0 @ GradientButtonMorph buttonHeight. 
	self addMorph: tm. 
	self extent: bounds extent.
! !

!WorkspacePane methodsFor: 'menus' stamp: 'jlb 3/26/2000 22:40'!
addModelItemsToWindowMenu: aMenu
	aMenu addLine.
	aMenu add: 'save contents to file...' target: self action: #saveContentsInFile. 
	aMenu add: 'use default background' target: self action: #setDefaultBackground.
	aMenu add: 'set edit text color' target: self action: #setEditTextColor.
! !

!WorkspacePane methodsFor: 'menus' stamp: 'jlb 3/24/2000 14:01'!
codePaneMenu: aMenu shifted: shifted
	"Note that unless we override perform:orSendTo:, PluggableTextController will respond to all menu items in a text pane"
	| donorMenu |
	donorMenu _ shifted
		ifTrue:
			[ParagraphEditor shiftedYellowButtonMenu]
		ifFalse:
			[ParagraphEditor yellowButtonMenu].
	^ aMenu labels: donorMenu labelString lines: donorMenu lineArray selections: donorMenu selections! !

!WorkspacePane methodsFor: 'menus' stamp: 'jlb 3/25/2000 19:17'!
saveContentsInFile
	| fileName stringToSave labelToUse |
	stringToSave _ (self dependents detect: [:dep | dep isKindOf: PluggableTextMorph] ifNone: [^ self beep]) text string.
	stringToSave size == 0 ifTrue: [^ self inform: 'nothing to save.'].
	labelToUse _  (self dependents detect: [:dep | dep isKindOf: SystemWindow] ifNone: [^ self beep]) label.

	fileName _ FillInTheBlank request: 'File name? (".text" will be added to end)' 
			initialAnswer: labelToUse.
	fileName size == 0 ifTrue: [^ self beep].
	(fileName asLowercase endsWith: '.text') ifFalse: [fileName _ fileName,'.text'].

	(FileStream newFileNamed: fileName)
		nextPutAll: stringToSave; close! !

!WorkspacePane methodsFor: 'private' stamp: 'jlb 3/28/2000 20:21'!
privateMoveBy: delta
	"Private!! Use 'position:' instead."
	| trailMorph start fill tm |
	(extension == nil or: [extension player == nil]) ifFalse:
		["Most cases eliminated fast by above test"
		self getPenDown ifTrue:
			["If this is a costume for a player with its pen down, draw a line."
			(trailMorph _ self trailMorph) ifNotNil:
				[start _ self referencePosition.
				trailMorph drawPenTrailFor: self from: start to: start + delta]]].
	fullBounds == bounds
		ifTrue: ["optimization: avoids recomputing fullBounds"
				fullBounds _ bounds _ bounds translateBy: delta]
		ifFalse: [bounds _ bounds translateBy: delta.
				fullBounds _ nil].
	(tm _ self findA: PluggableTextMorph)
		ifNil: [ ^ self ].
	fill _ tm fillStyle.
	fill isOrientedFill ifTrue:[fill origin: 1@1].! !

!WorkspacePane methodsFor: 'workspace stuff' stamp: 'jlb 3/25/2000 12:05'!
acceptContents: aString 
	"Set aString to be the contents of the receiver.  Return true cuz happy"

	self contents: aString.
	^ true! !

!WorkspacePane methodsFor: 'workspace stuff' stamp: 'jlb 3/25/2000 11:28'!
defaultContents

	^''! !

!WorkspacePane methodsFor: 'workspace stuff' stamp: 'jlb 3/25/2000 11:28'!
selectedMessageName

	^ nil! !


!WorkspacePane class reorganize!
('background' defaultBackground defaultBackground:)
('instance creation' openInWindow openInWindow:textColor:)
!


!WorkspacePane class methodsFor: 'background' stamp: 'jlb 3/25/2000 18:52'!
defaultBackground
	^ BackgroundForm! !

!WorkspacePane class methodsFor: 'background' stamp: 'jlb 3/25/2000 19:08'!
defaultBackground: theForm
	BackgroundForm _ theForm.! !

!WorkspacePane class methodsFor: 'instance creation' stamp: 'jlb 3/28/2000 21:40'!
openInWindow
	"WorkspacePane openInWindow"
     "Answer an Morphic view "
	| window wp |
	wp _ WorkspacePane new.
	window _ (SystemWindow labelled: 'Fancy Workspace') model: wp.
	window addMorph: wp frame: (0@0 corner: 1@1).
	wp color: (Preferences windowColorFor: #StringHolder).
	window updatePaneColors.
	window minimumExtent: 484 @ 200.
	^ window openInWorldExtent: 484@400	! !

!WorkspacePane class methodsFor: 'instance creation' stamp: 'jlb 3/28/2000 21:40'!
openInWindow: backgroundForm textColor: theColor
	"WorkspacePane openInWindow: WorkspacePane defaultBackground textColor: Color white"
     "Answer an Morphic view "
	| window wp |
	wp _ WorkspacePane new.
	window _ (SystemWindow labelled: 'Fancy Workspace') model: wp.
	wp setBackground: backgroundForm.
	wp textColor: theColor.
	window addMorph: wp frame: (0@0 corner: 1@1).
	wp color: (Preferences windowColorFor: #StringHolder).
	window updatePaneColors.
	window minimumExtent: 484 @ 200.
	^ window openInWorldExtent: 484@400	! !


GradientButtonMorph initialize!
WorkspacePane removeSelector: #privateMoveBy1:!
WorkspacePane removeSelector: #privateMoveBy2:!
WorkspacePane removeSelector: #position:!
WorkspacePane removeSelector: #setBackground1:!
WorkspacePane removeSelector: #defaultBackgroundColor!
WorkspacePane class removeSelector: #initialize!
