'From Squeak2.6 of 11 October 1999 [latest update: #1559] on 20 November 1999 at 5:13:12 pm'!
"Change Set:		IOHandle-part-1
Date:			14 November 1999
Author:			David T. Lewis

IOHandle adds classes which represent input/output on external
channels, such as files or sockets. The FileStream hierarchy is
modified to refer to instances of IOHandle rather than to raw
byte arrays when invoking primitives.

This change set must be loaded in two parts. The first part
creates a temporary FileStream hierachy to be used while the
real one is being updated.

Part 1 of this two part change set must be loaded before part 2.

This is pre-alpha experimental code. Please use it only on an
image you don't care very much about."!

FileStream subclass: #TempStandardFileStream
	instanceVariableNames: 'name fileID buffer1 '
	classVariableNames: 'Registry '
	poolDictionaries: ''
	category: 'OS-Support'!
TempStandardFileStream subclass: #TempCrLfFileStream
	instanceVariableNames: 'lineEndConvention '
	classVariableNames: 'Cr CrLf Lf LineEndDefault LineEndStrings LookAheadCount '
	poolDictionaries: ''
	category: 'OS-Support'!
TempCrLfFileStream subclass: #TempHtmlFileStream
	instanceVariableNames: 'prevPreamble '
	classVariableNames: 'TabThing '
	poolDictionaries: ''
	category: 'OS-Support'!
TempStandardFileStream class
	instanceVariableNames: ''!
TempCrLfFileStream class
	instanceVariableNames: ''!
TempHtmlFileStream class
	instanceVariableNames: ''!

!TempStandardFileStream commentStamp: 'dtl 11/12/1999 20:46' prior: 0!
This is a copy of class StandardFileStream. Please remove me from the system. -dtl!

!TempStandardFileStream methodsFor: 'open/close' stamp: 'ar 10/7/1998 14:43'!
close
	"Close this file."

	fileID ifNotNil: [
		self primClose: fileID.
		self unregister.
		fileID _ nil].
! !

!TempStandardFileStream methodsFor: 'open/close' stamp: 'jm 9/19/1998 17:01'!
closed
	"Answer true if this file is closed."

	^ fileID isNil
! !

!TempStandardFileStream methodsFor: 'open/close' stamp: 'jm 9/21/1998 16:20'!
ensureOpen
	"Make sure that this file really is open."

	self closed ifTrue: [^ self reopen].
	(self primSizeNoError: fileID) ifNotNil: [^ self].
	self reopen.
! !

!TempStandardFileStream methodsFor: 'open/close'!
open
	"For compatibility with a few existing things.  2/14/96 sw"

	^ self reopen! !

!TempStandardFileStream methodsFor: 'open/close' stamp: 'ar 10/7/1998 14:44'!
open: fileName forWrite: writeMode 
	"Open the file with the given name. If writeMode is true, allow writing, otherwise open the file in read-only mode."
	"Changed to do a GC and retry before failing ar 3/21/98 17:25"
	fileID _ self retryWithGC:[self primOpen: fileName writable: writeMode] until:[:id| id notNil].
	fileID ifNil: [^ nil].  "allows sender to detect failure"
	self register.
	name _ fileName.
	rwmode _ writeMode.
	buffer1 _ String new: 1.
! !

!TempStandardFileStream methodsFor: 'open/close'!
openReadOnly
	"Open the receiver as a read-only file.  1/31/96 sw"

	^ self open: name forWrite: false! !

!TempStandardFileStream methodsFor: 'open/close' stamp: 'jm 9/21/1998 13:58'!
reopen
	"Close and reopen this file. The file position is reset to zero."
	"Details: Files that were open when a snapshot occurs are no longer valid when the snapshot is resumed. This operation re-opens the file if that has happened."

	fileID ifNotNil: [self primCloseNoError: fileID].
	self open: name forWrite: rwmode.
! !

!TempStandardFileStream methodsFor: 'properties-setting'!
asHtml
	"Convert me in to an HtmlFileStream. 4/11/96 tk"

	^ self as: HtmlFileStream 
! !

!TempStandardFileStream methodsFor: 'properties-setting'!
ascii
	"opposite of binary"
	buffer1 _ String new: 1! !

!TempStandardFileStream methodsFor: 'properties-setting'!
binary
	buffer1 _ ByteArray new: 1! !

!TempStandardFileStream methodsFor: 'properties-setting'!
insertLineFeeds
	"(FileStream oldFileNamed: 'BBfix2.st') insertLineFeeds"
	| s crLf f |
	crLf _ String with: Character cr with: (Character value: 10).
	s _ ReadStream on: (self next: self size).
	self close.
	f _ FileStream newFileNamed: self name.
	[s atEnd] whileFalse: 
		[f nextPutAll: (s upTo: Character cr); nextPutAll: crLf].
	f close! !

!TempStandardFileStream methodsFor: 'properties-setting'!
isBinary
	^ buffer1 class == ByteArray! !

!TempStandardFileStream methodsFor: 'properties-setting' stamp: 'tk 11/4/1998 19:17'!
isReadOnly

	^ rwmode not
! !

!TempStandardFileStream methodsFor: 'properties-setting' stamp: 'jm 9/21/1998 13:56'!
readOnly
	"Make this file read-only."

	rwmode _ false.
! !

!TempStandardFileStream methodsFor: 'properties-setting' stamp: 'jm 9/21/1998 13:56'!
readWrite
	"Make this file writable."

	rwmode _ true.
! !

!TempStandardFileStream methodsFor: 'properties-setting' stamp: 'jm 12/5/97 15:14'!
setFileTypeToObject
	"On the Macintosh, set the file type and creator of this file to be a Squeak object file. On other platforms, do nothing. Setting the file type allows Squeak object files to be sent as email attachments and launched by double-clicking. On other platforms, similar behavior is achieved by creating the file with the '.sqo' file name extension."

	FileDirectory default
		setMacFileNamed: self fullName
		type: 'SOBJ'
		creator: 'FAST'.
! !

!TempStandardFileStream methodsFor: 'access' stamp: 'jm 9/21/1998 14:16'!
directory
	"Return the directory containing this file."

	^ FileDirectory forFileName: self fullName
! !

!TempStandardFileStream methodsFor: 'access'!
file
	"Answer the object representing the receiver's file.  Need for compatibility with some calls -- check senders.  2/14/96 sw"

	^ self! !

!TempStandardFileStream methodsFor: 'access' stamp: 'jm 9/21/1998 14:19'!
fullName
	"Answer this file's full path name."

	^ name
! !

!TempStandardFileStream methodsFor: 'access'!
isDirectory
	"Answer whether the receiver represents a directory.  For the post-transition case, uncertain what to do.  2/14/96 sw"
	^ false! !

!TempStandardFileStream methodsFor: 'access' stamp: 'ar 11/24/1998 14:00'!
localName
	^ name ifNotNil: [(name findTokens: FileDirectory pathNameDelimiter asString) last]! !

!TempStandardFileStream methodsFor: 'access' stamp: 'jm 9/21/1998 14:19'!
name
	"Answer this file's full path name."

	^ name
! !

!TempStandardFileStream methodsFor: 'access' stamp: 'di 6/27/97 12:18'!
peekFor: item 
	"Answer false and do not advance if the next element is not equal to item, or if this stream is at the end.  If the next element is equal to item, then advance over it and return true"
	| next |
	"self atEnd ifTrue: [^ false]. -- SFStream will give nil"
	(next _ self next) == nil ifTrue: [^ false].
	item = next ifTrue: [^ true].
	self skip: -1.
	^ false! !

!TempStandardFileStream methodsFor: 'access'!
printOn: aStream
	"Put a printed version of the receiver onto aStream.  1/31/96 sw"

	aStream nextPutAll: self class name; nextPutAll: ': '; print: name! !

!TempStandardFileStream methodsFor: 'access'!
reset
	^ self reopen! !

!TempStandardFileStream methodsFor: 'access'!
size
	"Answer the size of the file in characters.  2/12/96 sw"

	^ self primSize: fileID! !

!TempStandardFileStream methodsFor: 'read, write, position' stamp: 'sw 2/12/96'!
atEnd
	"Answer whether the receiver is at its end.  "
	^ self primAtEnd: fileID! !

!TempStandardFileStream methodsFor: 'read, write, position'!
flush
	"In some OS's seeking to 0 and back will do a flush"
	| p |
	p _ self position.
	self position: 0; position: p! !

!TempStandardFileStream methodsFor: 'read, write, position' stamp: 'jm 9/21/1998 14:11'!
next
	"Answer the next byte from this file, or nil if at the end of the file."

	| count |
	count _ self primRead: fileID into: buffer1 startingAt: 1 count: 1.
	count = 1
		ifTrue: [^ buffer1 at: 1]
		ifFalse: [^ nil].
! !

!TempStandardFileStream methodsFor: 'read, write, position'!
next: n
	"Return a string with the next n characters of the filestream in it.  1/31/96 sw"
	^ self nextInto: (buffer1 class new: n)! !

!TempStandardFileStream methodsFor: 'read, write, position' stamp: 'ar 2/5/1999 23:17'!
next: n into: aString
	"Read n bytes into the given string. Return the number of bytes read."
	^self primRead: fileID into: aString
		startingAt: 1 count: n! !

!TempStandardFileStream methodsFor: 'read, write, position'!
nextInto: aString
	"Fill aString, whose size dictates the size of the read, with characters from the receiver.  1/31/96 sw"
	| count wanted |
	count _ self primRead: fileID into: aString
				startingAt: 1 count: (wanted _ aString size).
	count < wanted ifTrue: [^ aString copyFrom: 1 to: count].
	^ aString! !

!TempStandardFileStream methodsFor: 'read, write, position' stamp: 'jm 9/21/1998 13:55'!
nextPut: char
	"Write the given character to this file."

	rwmode ifFalse: [^ self error: 'Cannot write a read-only file'].
	buffer1 at: 1 put: char.
	self primWrite: fileID from: buffer1 startingAt: 1 count: 1.
	^ char
! !

!TempStandardFileStream methodsFor: 'read, write, position' stamp: 'jm 9/21/1998 13:55'!
nextPutAll: aString
	"Write all the characters of the given string to this file."

	rwmode ifFalse: [^ self error: 'Cannot write a read-only file'].
	self primWrite: fileID from: aString startingAt: 1 count: aString size.
	^ aString
! !

!TempStandardFileStream methodsFor: 'read, write, position' stamp: 'tk 10/15/1998 09:30'!
padToEndWith: aChar
	"On the Mac, files do not truncate.  One can delete the old file and write a new one, but sometime deletion fails (file still open? file stale?).  This is a sad compromise.  Just let the file be the same length but pad it with a harmless character."

	| pad |
	self atEnd ifTrue: [^ self].
	pad _ self isBinary 
		ifTrue: [aChar asCharacter asciiValue]	"ok for char or number"
		ifFalse: [aChar asCharacter].
	self nextPutAll: (buffer1 class new: ((self size - self position) min: 20000) 
							withAll: pad).! !

!TempStandardFileStream methodsFor: 'read, write, position' stamp: 'sw 1/31/96'!
peek
	"Answer what would be returned if the message next were sent to the receiver. If the receiver is at the end, answer nil.  "
	| next |
	self atEnd ifTrue: [^ nil].
	next _ self next.
	self position: self position - 1.
	^ next! !

!TempStandardFileStream methodsFor: 'read, write, position'!
position
	"Return the receiver's current file position.  2/12/96 sw"

	^ self primGetPosition: fileID! !

!TempStandardFileStream methodsFor: 'read, write, position'!
position: pos
	"Set the receiver's position as indicated.  2/12/96 sw"

	^ self primSetPosition: fileID to: pos! !

!TempStandardFileStream methodsFor: 'read, write, position'!
readInto: byteArray startingAt: startIndex count: count
	"Read into the given array as specified, and return the count
	actually transferred.  index and count are in units of bytes or
	longs depending on whether the array is Bitmap, String or ByteArray"
	^ self primRead: fileID into: byteArray
			startingAt: startIndex count: count
! !

!TempStandardFileStream methodsFor: 'read, write, position'!
readOnlyCopy
	^ TempStandardFileStream readOnlyFileNamed: self name! !

!TempStandardFileStream methodsFor: 'read, write, position'!
setToEnd
	"Set the position of the receiver to the end of file.  1/31/96 sw"

	self position: self size! !

!TempStandardFileStream methodsFor: 'read, write, position'!
skip: n
	"Set the character position to n characters from the current position.
	Error if not enough characters left in the file.  1/31/96 sw"

	self position: self position + n! !

!TempStandardFileStream methodsFor: 'read, write, position' stamp: 'di 7/14/97 23:15'!
upTo: delim 
	"Fast version to speed up nextChunk"
	| pos buffer count |
	pos _ self position.
	buffer _ self next: 2000.
	(count _ buffer indexOf: delim) > 0 ifTrue: 
		["Found the delimiter part way into buffer"
		self position: pos + count.
		^ buffer copyFrom: 1 to: count - 1].
	self atEnd ifTrue:
		["Never found it, and hit end of file"
		^ buffer].
	"Never found it, but there's more..."
	^ buffer , (self upTo: delim)! !

!TempStandardFileStream methodsFor: 'read, write, position' stamp: 'jm 9/21/1998 13:56'!
verbatim: aString
	"A version of nextPutAll that can be called knowing it won't call nextPut: "

	^ self nextPutAll: aString
! !

!TempStandardFileStream methodsFor: 'primitives' stamp: 'jm 9/21/1998 16:18'!
primAtEnd: id
	"Answer true if the file position is at the end of the file."

	<primitive: 150>
	self primitiveFailed
! !

!TempStandardFileStream methodsFor: 'primitives' stamp: 'jm 9/21/1998 14:02'!
primClose: id
	"Close this file."

	<primitive: 151>
	self primitiveFailed
! !

!TempStandardFileStream methodsFor: 'primitives' stamp: 'jm 9/21/1998 14:02'!
primCloseNoError: id
	"Close this file. Don't raise an error if the primitive fails."

	<primitive: 151>
! !

!TempStandardFileStream methodsFor: 'primitives' stamp: 'jm 9/21/1998 14:04'!
primGetPosition: id
	"Get this files current position."

	<primitive: 152>
	self primitiveFailed
! !

!TempStandardFileStream methodsFor: 'primitives' stamp: 'jm 9/21/1998 14:06'!
primOpen: fileName writable: writableFlag
	"Open a file of the given name, and return the file ID obtained.
	If writableFlag is true, then
		if there is none with this name, then create one
		else prepare to overwrite the existing from the beginning
	otherwise
		if the file exists, open it read-only
		else return nil"

	<primitive: 153>
	^ nil
! !

!TempStandardFileStream methodsFor: 'primitives' stamp: 'jm 9/21/1998 16:18'!
primRead: id into: byteArray startingAt: startIndex count: count
	"Read up to count bytes of data from this file into the given string or byte array starting at the given index. Answer the number of bytes actually read."

	<primitive: 154>
	self closed ifTrue: [^ self error: 'File is closed'].
	self error: 'File read failed'.
! !

!TempStandardFileStream methodsFor: 'primitives' stamp: 'jm 9/21/1998 14:09'!
primSetPosition: id to: anInteger
	"Set this file to the given position."

	<primitive: 155>
	self primitiveFailed
! !

!TempStandardFileStream methodsFor: 'primitives' stamp: 'jm 9/21/1998 16:17'!
primSize: id
	"Answer the size of this file."

	<primitive: 157>
	self primitiveFailed
! !

!TempStandardFileStream methodsFor: 'primitives' stamp: 'jm 9/21/1998 16:17'!
primSizeNoError: id
	"Answer the size of this file. Answer nil if the primitive fails; this indicates that the file handle has become stale."

	<primitive: 157>
	^ nil
! !

!TempStandardFileStream methodsFor: 'primitives' stamp: 'jm 9/21/1998 16:18'!
primWrite: id from: stringOrByteArray startingAt: startIndex count: count
	"Write count bytes onto this file from the given string or byte array starting at the given index. Answer the number of bytes written."

	<primitive: 158>
	self closed ifTrue: [^ self error: 'File is closed'].
	self error: 'File write failed'.
! !

!TempStandardFileStream methodsFor: 'registry' stamp: 'ar 3/21/98 17:23'!
register
	^self class register: self! !

!TempStandardFileStream methodsFor: 'registry' stamp: 'ar 3/21/98 17:23'!
unregister
	^self class unregister: self! !

!TempStandardFileStream methodsFor: 'finalization' stamp: 'ar 3/21/98 18:16'!
actAsExecutor
	super actAsExecutor.
	name := nil.! !

!TempStandardFileStream methodsFor: 'finalization' stamp: 'ar 10/7/1998 15:44'!
finalize
	self primCloseNoError: fileID.! !


!TempCrLfFileStream commentStamp: 'dtl 11/12/1999 20:46' prior: 0!
This is a copy of class CrLfFileStream. Please remove me from the system. -dtl!

!TempCrLfFileStream methodsFor: 'open/close' stamp: 'ar 1/20/98 16:15'!
open: aFileName forWrite: writeMode 
	"Open the receiver.  If writeMode is true, allow write, else access will be 
	read-only. "
	| result |
	result _ super open: aFileName forWrite: writeMode.
	result ifNotNil: [self detectLineEndConvention].
	^ result! !

!TempCrLfFileStream methodsFor: 'access' stamp: 'ar 1/20/98 16:16'!
ascii
	super ascii.
	self detectLineEndConvention! !

!TempCrLfFileStream methodsFor: 'access' stamp: 'ar 1/20/98 16:16'!
binary
	super binary.
	lineEndConvention _ nil! !

!TempCrLfFileStream methodsFor: 'access' stamp: 'ls 7/10/1998 23:35'!
detectLineEndConvention
	"Detect the line end convention used in this stream. The result may be either #cr, #lf or #crlf."
	| char numRead pos |
	self isBinary ifTrue: [^ self error: 'Line end conventions are not used on binary streams'].
	lineEndConvention _ LineEndDefault.
	"Default if nothing else found"
	numRead _ 0.
	pos _ super position.
	[super atEnd not and: [numRead < LookAheadCount]]
		whileTrue: 
			[char _ super next.
			char = Lf
				ifTrue: 
					[super position: pos.
					^ lineEndConvention _ #lf].
			char = Cr
				ifTrue: 
					[super peek = Lf
						ifTrue: [lineEndConvention _ #crlf]
						ifFalse: [lineEndConvention _ #cr].
					super position: pos.
					^ lineEndConvention].
			numRead _ numRead + 1].
	super position: pos.
	^ lineEndConvention! !

!TempCrLfFileStream methodsFor: 'access' stamp: 'ls 11/5/1998 23:37'!
next
    | char secondChar |
    char _ super next.
    self isBinary ifTrue: [^char].
    char == Cr ifTrue:
        [secondChar _ super next.
        secondChar ifNotNil: [secondChar == Lf ifFalse: [self skip: -1]].
        ^Cr].
    char == Lf ifTrue: [^Cr].
    ^char! !

!TempCrLfFileStream methodsFor: 'access' stamp: 'ls 12/29/1998 17:15'!
next: n

		| string peekChar |
		string _ super next: n.
		string size = 0 ifTrue: [ ^string ].
		self isBinary ifTrue: [ ^string ].

		"if we just read a CR, and the next character is an LF, then skip the LF"
		( string last = Character cr ) ifTrue: [
			peekChar _ super next.		"super peek doesn't work because it relies on #next"
			peekChar ~= Character lf ifTrue: [
				super position: (super position - 1) ]. ].
 
		string _ string withSqueakLineEndings.

		string size = n ifTrue: [ ^string ].

		"string shrunk due to embedded crlfs; make up the difference"
		^string, (self next: n - string size)! !

!TempCrLfFileStream methodsFor: 'access' stamp: 'ar 1/20/98 16:18'!
nextPut: char 
	(lineEndConvention notNil and: [char = Cr])
		ifTrue: [super nextPutAll: (LineEndStrings at: lineEndConvention)]
		ifFalse: [super nextPut: char].
	^ char! !

!TempCrLfFileStream methodsFor: 'access' stamp: 'ar 1/20/98 16:18'!
nextPutAll: aString 
	super nextPutAll: (self convertStringFromCr: aString).
	^ aString
! !

!TempCrLfFileStream methodsFor: 'access' stamp: 'wod 6/18/1998 13:52'!
peek
	"Answer what would be returned if the message next were sent to the receiver. If the receiver is at the end, answer nil.  "
	| next pos |
	self atEnd ifTrue: [^ nil].
	pos _ self position.
	next _ self next.
	self position: pos.
	^ next! !

!TempCrLfFileStream methodsFor: 'access' stamp: 'wod 11/5/1998 14:15'!
upTo: aCharacter
	| newStream char |
	newStream _ WriteStream on: (String new: 100).
	[(char _ self next) isNil or: [char == aCharacter]]
		whileFalse: [newStream nextPut: char].
	^ newStream contents
! !

!TempCrLfFileStream methodsFor: 'access' stamp: 'ar 1/20/98 16:18'!
verbatim: aString 
	super verbatim: (self convertStringFromCr: aString).
	^ aString! !

!TempCrLfFileStream methodsFor: 'private' stamp: 'ar 1/20/98 16:21'!
convertStringFromCr: aString 
	| inStream outStream |
	lineEndConvention ifNil: [^ aString].
	lineEndConvention == #cr ifTrue: [^ aString].
	lineEndConvention == #lf ifTrue: [^ aString copy replaceAll: Cr with: Lf].
	"lineEndConvention == #crlf"
	inStream _ ReadStream on: aString.
	outStream _ WriteStream on: (String new: aString size).
	[inStream atEnd]
		whileFalse: 
			[outStream nextPutAll: (inStream upTo: Cr).
			(inStream atEnd not or: [aString last = Cr])
				ifTrue: [outStream nextPutAll: CrLf]].
	^ outStream contents! !

!TempCrLfFileStream methodsFor: 'private' stamp: 'ar 1/20/98 16:21'!
convertStringToCr: aString 
	| inStream outStream |
	lineEndConvention ifNil: [^ aString].
	lineEndConvention == #cr ifTrue: [^ aString].
	lineEndConvention == #lf ifTrue: [^ aString copy replaceAll: Lf with: Cr].
	"lineEndConvention == #crlf"
	inStream _ ReadStream on: aString.
	outStream _ WriteStream on: (String new: aString size).
	[inStream atEnd]
		whileFalse: 
			[outStream nextPutAll: (inStream upTo: Cr).
			(inStream atEnd not or: [aString last = Cr])
				ifTrue: 
					[outStream nextPut: Cr.
					inStream peek = Lf ifTrue: [inStream next]]].
	^ outStream contents! !


!TempHtmlFileStream commentStamp: '<historical>' prior: 0!
The Class apes StandardFileStream, but converts the text to HTML before putting it out (primarily intended for printOut).  It can be invoked with

	((FileStream fileNamed: 'changes.html') asHtml) fileOutChanges

Use usual FileStream methods to put out text converted to
	HTML fairly approximating that text  (for best looks, use 
	method:, methodHeader:, methodBody:, for code);

verbatim: puts text out without conversion;

command: put out HTML items, such as <br>, supplying the brackets.

header: and trailer: put out an HTML wrapper (preamble and closing text)

nextPut does the actual conversion, nextPutAll: defers characters to nextPut.

The code is fairly dumb at present, doing a wooden straightforward conversion of the text without attempting to capture the style or fonts in which the original text was rendered.  Tabs are handled awkwardly, using &nbsp, so that probably only leading strings are working right.  Style sheets now permit us to do a much neater looking job if there is interest in improving the looks of things.!
]style[(1039)f1cred;!

!TempHtmlFileStream methodsFor: 'HTML'!
command: aString
	"Append HTML commands directly without translation.  Caller should not include < or >.  Note that font change info comes through here!!  4/5/96 tk"

	(aString includes: $<) ifTrue: [self error: 'Do not put < or > in arg'].
		"We do the wrapping with <> here!!  Don't put it in aString."
	^ self verbatim: '<', aString, '>'! !

!TempHtmlFileStream methodsFor: 'fileIn/Out' stamp: 'di 6/28/97 08:35'!
copyMethodChunkFrom: aStream
	"Overridden to bolden the first line (presumably a method header)"
	| terminator code firstLine |
	terminator _ $!!.
	aStream skipSeparators.
	code _ aStream upTo: terminator.
	firstLine _ code copyUpTo: Character cr.
	firstLine size = code size
		ifTrue: [self nextPutAll: code]
		ifFalse: [self command: 'b'; nextPutAll: firstLine; command: '/b'.
				self nextPutAll: (code copyFrom: firstLine size + 1 to: code size)].
	self nextPut: terminator.
	[aStream peekFor: terminator] whileTrue:   "case of imbedded (doubled) terminators"
			[self nextPut: terminator;
				nextPutAll: (aStream upTo: terminator);
				nextPut: terminator]! !

!TempHtmlFileStream methodsFor: 'fileIn/Out' stamp: 'di 6/28/97 09:21'!
copyPreamble: preamble from: aStream
	"Make method category preambles bold at category changes."

	prevPreamble = preamble
		ifTrue: [super copyPreamble: preamble from: aStream]
		ifFalse: [self command: 'H3'.
				super copyPreamble: preamble from: aStream.
				self command: '/H3'.
				prevPreamble _ preamble]! !

!TempHtmlFileStream methodsFor: 'fileIn/Out' stamp: 'di 6/28/97 21:43'!
nextChunk
	"Answer the contents of the receiver, up to the next terminator character (!!).  Imbedded terminators are doubled.  Undo and strip out all Html stuff in the stream and convert the characters back.  4/12/96 tk"
	| out char did rest |
	self skipSeparators.	"Absorb <...><...> also"
	out _ WriteStream on: (String new: 500).
	[self atEnd] whileFalse: [
		self peek = $< ifTrue: [self unCommand].	"Absorb <...><...>"
		(char _ self next) = $&
			ifTrue: [
				rest _ self upTo: $;.
				did _ out position.
				rest = 'lt' ifTrue: [out nextPut: $<].
				rest = 'gt' ifTrue: [out nextPut: $>].
				rest = 'amp' ifTrue: [out nextPut: $&].
				did = out position ifTrue: [
					self error: 'new HTML char encoding'.
					"Please add it to this code"]]
			ifFalse: [char = $!!	"terminator"
				ifTrue: [
					self peek = $!! ifFalse: [^ out contents].
					out nextPut: self next]	"pass on one $!!"
				ifFalse: [char asciiValue = 9
							ifTrue: [self next; next; next; next "TabThing"].
						out nextPut: char]]
		].
	^ out contents! !

!TempHtmlFileStream methodsFor: 'fileIn/Out'!
skipSeparators
	"Bsides the normal spacers, also skip any <...>, html commands.
	4/12/96 tk"
	| did |
	[did _ self position.
		super skipSeparators.
		self unCommand.	"Absorb <...><...>"
		did = self position] whileFalse.	"until no change"
! !

!TempHtmlFileStream methodsFor: 'read, write, position' stamp: 'di 6/28/97 21:49'!
header
	"append the HTML header.  Be sure to call trailer after you put out the data.
	4/4/96 tk"
	| cr |
	cr _ String with: Character cr.
	self command: 'HTML'; verbatim: cr.
	self command: 'HEAD'; verbatim: cr.
	self command: 'TITLE'.
	self nextPutAll: '"', self name, '"'.
	self command: '/TITLE'; verbatim: cr.
	self command: '/HEAD'; verbatim: cr.
	self command: 'BODY'; verbatim: cr.
! !

!TempHtmlFileStream methodsFor: 'read, write, position' stamp: 'acg 01/01/1999 13:59'!
nextPut: char
	"Put a character on the file, but translate it first. 4/6/96 tk 1/1/98 acg"
	char = $< ifTrue: [^ super nextPutAll: '&lt;'].
	char = $> ifTrue: [^ super nextPutAll: '&gt;'].
	char = $& ifTrue: [^ super nextPutAll: '&amp;'].
	char asciiValue = 13 "return" 
		ifTrue: [self command: 'br'].
	char = $	"tab" 
		ifTrue: [self verbatim: TabThing. ^super nextPut: char].
	^ super nextPut: char! !

!TempHtmlFileStream methodsFor: 'read, write, position'!
nextPutAll: aString
	"Write the whole string, translating as we go. 4/6/96 tk"
	"Slow, but faster than using aString asHtml?"

	aString do: [:each | self nextPut: each].! !

!TempHtmlFileStream methodsFor: 'read, write, position' stamp: 'acg 01/07/1999 09:24'!
trailer
	"append the HTML trailer.  Call this just before file close.
	4/4/96 tk"
	| cr |
	cr _ String with: Character cr.
	self command: '/BODY'; verbatim: cr.
	self command: '/HTML'; verbatim: cr.
!
]style[(7 192)f1bcblue;,f1! !

!TempHtmlFileStream methodsFor: 'read, write, position' stamp: 'acg 01/02/1999 00:38'!
verbatim: aString
	"Put out the string without HTML conversion. 1/1/99 acg"

	super nextPutAll: aString

	"'super verbatim:' in the 2.3beta draft didn't perform as expected -- the code was printed with conversion.  In a sense, that wouldn't make sense either -- we don't want strictly verbatim printing, just printing without the HTML conversion (that is, skipping around just the nextPut: and nextPutAll: for just this Class).  If there were intermediate conversions (say, CRLF!!), we would want those to happen as advertised -- perhaps we should use a differently named selector, perhaps something like nextPutWithoutHTMLConversion:, so that verbatim isn't overridden?"! !


!TempStandardFileStream class methodsFor: 'file creation' stamp: 'jm 9/21/1998 13:13'!
fileNamed: fileName
	"Open a file with the given name for reading and writing. If the name has no directory part, then the file will be created in the default directory. If the file already exists, its prior contents may be modified or replaced, but the file will not be truncated on close."

	^ self new open: fileName forWrite: true
! !

!TempStandardFileStream class methodsFor: 'file creation' stamp: 'mpw 9/18/1999 00:05'!
isAFileNamed: fileName
	"Answer true if a file of the given name exists."

	| f |
	f _ self new open: fileName forWrite: false.
	f ifNil: [^ false].
	f close.
	^ true
! !

!TempStandardFileStream class methodsFor: 'file creation' stamp: 'jm 9/21/1998 13:14'!
newFileNamed: fileName
 	"Create a new file with the given name, and answer a stream opened for writing on that file. If the file already exists, ask the user what to do."

	| dir localName choice newName |
	(self isAFileNamed: fileName)
		ifFalse: [^ self new open: fileName forWrite: true].

	"file already exists:"
	dir _ FileDirectory forFileName: fileName.
	localName _ FileDirectory localNameFor: fileName.
	choice _ (PopUpMenu
		labels:
'overwrite that file
choose another name
cancel')
		startUpWithCaption: localName, '
already exists.'.

	choice = 1 ifTrue: [
		dir deleteFileNamed: localName
			ifAbsent: [self error: 'Could not delete the old version of that file'].
		^ self new open: fileName forWrite: true].

	choice = 2 ifTrue: [
		newName _ FillInTheBlank request: 'Enter a new file name' initialAnswer: fileName.
		^ self newFileNamed: newName].

	self error: 'Please close this to abort file opening'.
! !

!TempStandardFileStream class methodsFor: 'file creation' stamp: 'jm 9/21/1998 13:16'!
oldFileNamed: fileName
	"Open an existing file with the given name for reading and writing. If the name has no directory part, then the file will be created in the default directory. If the file already exists, its prior contents may be modified or replaced, but the file will not be truncated on close."

	| selection |
	(self isAFileNamed: fileName) ifTrue:
		[^ self new open: fileName forWrite: true].

	"File does not exist..."
	selection _ (PopUpMenu labels:
'create a new file
choose another name
cancel')
			startUpWithCaption: (FileDirectory localNameFor: fileName) , '
does not exist.'.
	selection = 1 ifTrue:
		[^ self new open: fileName forWrite: true].
	selection = 2 ifTrue:
		[^ self oldFileNamed:
			(FillInTheBlank request: 'Enter a new file name'
						initialAnswer: (FileDirectory localNameFor: fileName))].
	self halt! !

!TempStandardFileStream class methodsFor: 'file creation' stamp: 'jm 9/21/1998 13:19'!
readOnlyFileNamed: fileName
	"Open an existing file with the given name for reading."

	| selection |
	(self isAFileNamed: fileName) ifTrue:
		[^ self new open: fileName forWrite: false].

	"File does not exist..."
	selection _ (PopUpMenu labels: 'choose another name
cancel')
			startUpWithCaption: (FileDirectory localNameFor: fileName) , '
does not exist.'.
	selection = 1 ifTrue:
		[^ self readOnlyFileNamed:
			(FillInTheBlank request: 'Enter a new file name'
						initialAnswer: (FileDirectory localNameFor: fileName))].
	self halt! !

!TempStandardFileStream class methodsFor: 'registry' stamp: 'ar 10/7/1998 14:41'!
register: anObject
	WeakArray isFinalizationSupported ifFalse:[^anObject].
	self registry add: anObject! !

!TempStandardFileStream class methodsFor: 'registry' stamp: 'ar 10/7/1998 14:41'!
registry
	WeakArray isFinalizationSupported ifFalse:[^nil].
	^Registry isNil
		ifTrue:[Registry := WeakRegistry new]
		ifFalse:[Registry].! !

!TempStandardFileStream class methodsFor: 'registry' stamp: 'ar 10/7/1998 15:23'!
unregister: anObject
	WeakArray isFinalizationSupported ifFalse:[^anObject].
	self registry remove: anObject ifAbsent:[]! !


!TempCrLfFileStream class methodsFor: 'class initialization' stamp: 'ar 1/20/98 16:10'!
defaultToCR
	"TempCrLfFileStream defaultToCR"
	LineEndDefault := #cr.! !

!TempCrLfFileStream class methodsFor: 'class initialization' stamp: 'ar 1/20/98 16:10'!
defaultToCRLF
	"TempCrLfFileStream defaultToCRLF"
	LineEndDefault := #crlf.! !

!TempCrLfFileStream class methodsFor: 'class initialization' stamp: 'ar 1/20/98 16:10'!
defaultToLF
	"TempCrLfFileStream defaultToLF"
	LineEndDefault := #lf.! !

!TempCrLfFileStream class methodsFor: 'class initialization' stamp: 'ar 1/20/98 16:13'!
guessDefaultLineEndConvention
	"Lets try to guess the line end convention from what we know about the path name delimiter from FileDirectory."
	FileDirectory pathNameDelimiter = $: ifTrue:[^self defaultToCR].
	FileDirectory pathNameDelimiter = $/ ifTrue:[^self defaultToLF].
	FileDirectory pathNameDelimiter = $\ ifTrue:[^self defaultToCRLF].
	"in case we don't know"
	^self defaultToCR! !

!TempCrLfFileStream class methodsFor: 'class initialization' stamp: 'di 2/4/1999 09:16'!
initialize
	"TempCrLfFileStream initialize"
	Cr := Character cr.
	Lf := Character lf.
	CrLf := String with: Cr with: Lf.
	LineEndStrings := Dictionary new.
	LineEndStrings at: #cr put: (String with: Character cr).
	LineEndStrings at: #lf put: (String with: Character lf).
	LineEndStrings at: #crlf put: (String with: Character cr with: Character lf).
	LookAheadCount := 2048.
	Smalltalk addToStartUpList: self.
	self startUp.! !

!TempCrLfFileStream class methodsFor: 'class initialization' stamp: 'djp 1/28/1999 22:08'!
startUp
	self guessDefaultLineEndConvention! !


!TempHtmlFileStream class methodsFor: 'instance creation' stamp: 'acg 01/07/1999 09:24'!
newFrom: aFileStream
	"Answer an TempHtmlFileStream that is 'like' aFileStream.  As a side-effect, the surviving fileStream answered by this method replaces aFileStream on the finalization registry. 1/6/99 acg"

	|inst|
	inst _ super newFrom: aFileStream.
	StandardFileStream unregister: aFileStream.
	TempHtmlFileStream register: inst.
	inst detectLineEndConvention.
	^inst
! !

!TempHtmlFileStream class methodsFor: 'class initialization' stamp: 'acg 01/01/1999 13:57'!
initialize   "TempHtmlFileStream initialize"
	TabThing _ '&nbsp;&nbsp;&nbsp;'

"I took Ted's suggestion to use &nbsp, which works far better for the HTML.  Style sheets provide an alternative, possibly better, solution since they permit finer-grain control of the HTML formatting, and thus would permit capturing the style in which text was originally rendered.  Internal tabbings would still get lost. 1/1/99 acg."! !


TempCrLfFileStream initialize!
TempHtmlFileStream initialize!
"Postscript:

Save a copy of the current FileStream>>concreteStream method
by stashing it into the ATemporaryStandardFileStream class.
This is just a handy place to store it for the time being."

(Object subclass: #ATemporaryMethodSaver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OS-Support')
		class
		copy: #concreteStream
		from: FileStream class
		classified: 'concrete classes'.

"Now replace the method with a new one to use temporarily."
FileStream class compile: 'concreteStream
	^ TempCrLfFileStream'.

"Reopen the sources and changes files using the temporary class."
(SourceFiles at: 1) close.
SourceFiles at: 1 put: (FileStream oldFileNamed: (SourceFiles at: 1) name).
(SourceFiles at: 2) close.
SourceFiles at: 2 put: (FileStream oldFileNamed: (SourceFiles at: 2) name).


!

