'From Squeak2.6 of 11 October 1999 [latest update: #1559] on 20 November 1999 at 5:13:36 pm'!
"Change Set:		IOHandle-part-2-
Date:			14 November 1999
Author:			David T. Lewis

IOHandle part 2 of 2. Please load part 1 first."

!

Object subclass: #IOHandle
	instanceVariableNames: 'sessionID '
	classVariableNames: 'BigEndianMachine Registry '
	poolDictionaries: ''
	category: 'OS-Support'!
IOHandle subclass: #IOFileHandle
	instanceVariableNames: 'file writable fileSize lastOp sqFileStruct '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OS-Support'!
IOHandle subclass: #IOSocketHandle
	instanceVariableNames: 'socketType privateSocketPtr sqSocketStruct '
	classVariableNames: 'Connected InvalidSocket OtherEndClosed ThisEndClosed Unconnected WaitingForConnection '
	poolDictionaries: ''
	category: 'OS-Support'!
Object subclass: #Socket
	instanceVariableNames: 'semaphore ioSocketHandle '
	classVariableNames: 'Connected DeadServer InvalidSocket OtherEndClosed Registry TCPSocketType ThisEndClosed UDPSocketType Unconnected WaitingForConnection '
	poolDictionaries: ''
	category: 'System-Network'!
FileStream subclass: #StandardFileStream
	instanceVariableNames: 'name ioHandle buffer1 '
	classVariableNames: 'Registry '
	poolDictionaries: ''
	category: 'System-Files'!

!FileStream class methodsFor: 'instance creation' stamp: 'dtl 10/24/1999 21:47'!
name: aSymbolOrString attachTo: anIOHandle
	"Create a new instance attached to anIOHandle, where anIOHandle represents
	an open IO channel. For write streams, this represents two Smalltalk streams
	which write to the same OS file or output stream, presumably with interleaved
	output. The purpose of this method is to permit a FileStream to be attached to
	an existing IOHandle, such as the IOHandle for standard input, standard output,
	and standard error."

	^ self concreteStream name: aSymbolOrString attachTo: anIOHandle
! !


!IOHandle commentStamp: 'dtl 11/13/1999 20:06' prior: 0!
I represent an operating system input/output channel. My subclasses represent specific
kinds of IO channels, such as those for files or network connections.

I am responsible for primitive interfaces to an external object such as a file or network
socket. Other classes, such as FileStream subclasses, may use me for accessing these
primitives.

I am responsible for maintaining instance variables which reflect the current values of
my associated IO channels. These instance variables are intended to permit the state of an
IO channel to be examined in a browser.

Warnings:

My private methods use some machine dependent assumptions. This is a Bad Thing.
I (along with my subclasses) also contain knowledge of the specifics of some C language
data structures. This is a Very Bad Thing. Because of these indiscretions, this class
hierarchy as currently implemented may be a Bad Idea. Consider yourself warned.
(Note: A better implemention might eliminate the machine dependencies by using a few
extra primitives. However, I do not want this experimental version of IOHandle to be
dependent on additional primitives at this time.

I have not measured the performance impact of IOHandle on file I/O. I would expect
a negative performance impact due to the updating of instance variables after reads
or writes, but I have not bothered to measure it. I do not see any noticeable change in
the interactive feel of Squeak, or any noticeable change in the time required to save
images or do other normal file I/O. -dtl!

!IOHandle reorganize!
('accessing' sessionID sessionID:)
('registry' register unregister)
('private - converting' asSQFileStruct asSQSocketStruct byteArrayFromInteger: integerFromByteArray:)
!


!IOHandle methodsFor: 'accessing' stamp: 'dtl 9/19/1999 10:53'!
sessionID

	^ sessionID! !

!IOHandle methodsFor: 'accessing' stamp: 'dtl 10/28/1999 12:36'!
sessionID: aByteArray

	((aByteArray species = ByteArray) and: [ aByteArray size = 4 ])
		 ifFalse: [ self error: 'invalid argument' ].
	sessionID _ aByteArray! !

!IOHandle methodsFor: 'registry' stamp: 'dtl 11/13/1999 10:15'!
register

	^ self class register: self! !

!IOHandle methodsFor: 'registry' stamp: 'dtl 11/13/1999 10:18'!
unregister

	^ self class unregister: self! !

!IOHandle methodsFor: 'private - converting' stamp: 'dtl 11/12/1999 19:59'!
asSQFileStruct
	"Answer a ByteArray which can be treated as a struct SQFile by the virtual machine.
	This is an optimization for performance, in which the value of the SQFile ByteArray
	is held by my sqFileStruct instance variable."

	"FIXME: Do some testing to see if this really makes any difference. If not, eliminate
	the instance variable and this method."

	| struct |
	struct _ ByteArray new: self class sqFileStructureSize.
	struct replaceFrom: 1 to: 4 with: self file.
	struct replaceFrom: 5 to: 8 with: self sessionID.
	struct replaceFrom: 9 to: 12
		   with: (self byteArrayFromInteger: (self writable ifTrue: [1] ifFalse: [0])).
	struct replaceFrom: 13 to: 16 with: (self byteArrayFromInteger: self fileSize).
	struct replaceFrom: 17 to: 20 with: (self byteArrayFromInteger: self lastOp).
	^ struct


! !

!IOHandle methodsFor: 'private - converting' stamp: 'dtl 11/12/1999 20:00'!
asSQSocketStruct
	"Answer a ByteArray which can be treated as a struct SQSocket by the virtual machine.
	This is an optimization for performance, in which the value of the SQSocket ByteArray
	is held by my sqSocketStruct instance variable."

	"FIXME: Do some testing to see if this really makes any difference. If not, eliminate
	the instance variable and this method."

	| struct |
	struct _ ByteArray new: self class sqSocketStructureSize.
	struct replaceFrom: 1 to: 4 with: self sessionID.
	struct replaceFrom: 5 to: 8
		with: (self byteArrayFromInteger: (self class socketTypeAsInteger: self socketType)).
	struct replaceFrom: 9 to: 12 with: self privateSocketPtr.
	^ struct


! !

!IOHandle methodsFor: 'private - converting' stamp: 'dtl 11/2/1999 05:24'!
byteArrayFromInteger: anInteger
	"Convert anInteger into a four byte ByteArray in machine dependent byte ordering.
	Assumes 32 bit integer word size for this machine."

	| b |
	b _ ByteArray new: 4.
	self class bigEndianMachine
		ifTrue: [(WriteStream on: b) nextNumber: 4 put: anInteger]
		ifFalse: [(WriteStream on: b) nextLitteEndianNumber: 4 put: anInteger].
	^ b! !

!IOHandle methodsFor: 'private - converting' stamp: 'dtl 11/2/1999 05:23'!
integerFromByteArray: anArrayOfFourBytes
	"Convert a four byte ByteArray in machine dependent byte ordering to an integer.
	Assumes 32 bit integer word size for this machine."

	(anArrayOfFourBytes size = 4) ifFalse: [self error: 'invalid argument, should be four byte ByteArray'].
	^ anArrayOfFourBytes longAt: 1 bigEndian: self class bigEndianMachine
! !


!IOFileHandle commentStamp: 'dtl 10/30/1999 10:39' prior: 0!
I represent an IO channel on a file system object, such as a file. I keep a ByteArray
which corresponds to a SQFile data structure in the Squeak virtual machine. My
instance variables are updated to reflect the current value of this data structure,
allowing the data structure to be indirectly viewed with a normal Smalltalk inspector.
!

!IOFileHandle reorganize!
('accessing' file file: fileSize fileSize: lastOp lastOp: sessionID sqFileStruct sqFileStruct: writable writable:)
('open/close' close closeNoError open:forWrite:)
('read, write, position' atEnd flush position position: readInto:startingAt:count: size writeFrom:startingAt:count:)
('testing' isOpen)
('converting' asSQFileStruct)
('updating' changed: update:)
('printing' printOn:)
('private - converting' fileFromSqFileStruct fileSizeFromSqFileStruct lastOpFromSqFileStruct sessionIDFromSqFileStruct writableFromSqFileStruct)
('primitives' primAtEnd: primClose: primCloseNoError: primGetPosition: primOpen:writable: primRead:into:startingAt:count: primSetPosition:to: primSize: primSizeNoError: primWrite:from:startingAt:count:)
!


!IOFileHandle methodsFor: 'accessing' stamp: 'dtl 10/29/1999 10:51'!
file
	"Always update from the current value of sqFileStruct."

	^ file _ self fileFromSqFileStruct! !

!IOFileHandle methodsFor: 'accessing' stamp: 'dtl 10/28/1999 11:11'!
file: aByteArray

	((aByteArray species = ByteArray) and: [ aByteArray size = 4 ])
		 ifFalse: [ self error: 'invalid argument' ].
	file _ aByteArray! !

!IOFileHandle methodsFor: 'accessing' stamp: 'dtl 11/1/1999 21:55'!
fileSize
	"Always update from the current value of sqFileStruct."

	fileSize _ self fileSizeFromSqFileStruct! !

!IOFileHandle methodsFor: 'accessing' stamp: 'dtl 10/28/1999 11:11'!
fileSize: anInteger

	fileSize _ anInteger
! !

!IOFileHandle methodsFor: 'accessing' stamp: 'dtl 11/1/1999 21:57'!
lastOp
	"Always update from the current value of sqFileStruct."

	^ lastOp _ self lastOpFromSqFileStruct! !

!IOFileHandle methodsFor: 'accessing' stamp: 'dtl 11/19/1999 22:02'!
lastOp: aSymbolOrString
	"lastOp represents IO operations such as READ and WRITE."

	lastOp _ aSymbolOrString
! !

!IOFileHandle methodsFor: 'accessing' stamp: 'dtl 10/29/1999 10:45'!
sessionID
	"Always update from the current value of sqFileStruct."

	^ sessionID _ self sessionIDFromSqFileStruct! !

!IOFileHandle methodsFor: 'accessing' stamp: 'dtl 10/23/1999 11:40'!
sqFileStruct

	^ sqFileStruct! !

!IOFileHandle methodsFor: 'accessing' stamp: 'dtl 10/29/1999 11:39'!
sqFileStruct: aByteArray

	sqFileStruct _ aByteArray.
	self changed! !

!IOFileHandle methodsFor: 'accessing' stamp: 'dtl 11/1/1999 21:59'!
writable
	"Always update from the current value of sqFileStruct."

	^ writable _ self writableFromSqFileStruct! !

!IOFileHandle methodsFor: 'accessing' stamp: 'dtl 10/28/1999 12:09'!
writable: aBoolean
	"True if this IO channel is writable, false if read-only. In the SQFile data structure, this
	is represented as integer 1 for true or integer 0 for false."

	writable _ aBoolean! !

!IOFileHandle methodsFor: 'open/close' stamp: 'dtl 11/13/1999 10:04'!
close
	"Close this file."

	self primClose: sqFileStruct.
	self changed.
	self unregister
! !

!IOFileHandle methodsFor: 'open/close' stamp: 'dtl 11/13/1999 11:00'!
closeNoError
	"Close this file."

	self primCloseNoError: sqFileStruct.
	self changed.
	self unregister
! !

!IOFileHandle methodsFor: 'open/close' stamp: 'dtl 11/13/1999 12:00'!
open: fileName forWrite: writeMode 
	"Open the file with the given name. If writeMode is true, allow writing, otherwise
	open the file in read-only mode."
	"Changed to do a GC and retry before failing ar 3/21/98 17:25"
	"Moved this method from StandardFileStream to IOFileHandle. dtl 12-Nov-1999"

	sqFileStruct _ self retryWithGC: [self primOpen: fileName writable: writeMode]
					until:[:id| id notNil].
	sqFileStruct ifNil: [^ nil].  "allows sender to detect failure"
	self changed.
	self register

! !

!IOFileHandle methodsFor: 'read, write, position' stamp: 'dtl 11/13/1999 11:27'!
atEnd
	"Answer whether the receiver is at its end."

	^ self primAtEnd: sqFileStruct! !

!IOFileHandle methodsFor: 'read, write, position' stamp: 'dtl 11/13/1999 11:35'!
flush
	"In some OS's seeking to 0 and back will do a flush"
	"FIXME: Are there any OS's that cannot do an fflush()? This looks rather silly. -dtl"

	| p |
	p _ self position.
	self position: 0; position: p! !

!IOFileHandle methodsFor: 'read, write, position' stamp: 'dtl 11/13/1999 11:29'!
position
	"Return the receiver's current file position."

	^ self primGetPosition: sqFileStruct! !

!IOFileHandle methodsFor: 'read, write, position' stamp: 'dtl 11/13/1999 11:31'!
position: pos
	"Set the receiver's file position."

	^ self primSetPosition: sqFileStruct to: pos! !

!IOFileHandle methodsFor: 'read, write, position' stamp: 'dtl 11/19/1999 21:51'!
readInto: buffer startingAt: index count: num

	| numRead |
	numRead _ self primRead: sqFileStruct into: buffer startingAt: index count: num.
	self changed.	"FIXME: Check what this does to performance. -dtl"
	^ numRead! !

!IOFileHandle methodsFor: 'read, write, position' stamp: 'dtl 11/13/1999 12:36'!
size
	"Answer the size of the file in characters."

	^ self primSize: sqFileStruct! !

!IOFileHandle methodsFor: 'read, write, position' stamp: 'dtl 11/19/1999 21:52'!
writeFrom: buffer startingAt: index count: num

	| numWritten |
	self writable ifFalse: [self error: 'Cannot write a read-only file'. ^ 0].
	numWritten _ self primWrite: sqFileStruct from: buffer startingAt: index count: num.
	self changed.	"FIXME: Check what this does to performance. -dtl"
	^ numWritten
! !

!IOFileHandle methodsFor: 'testing' stamp: 'dtl 11/13/1999 11:13'!
isOpen
	"Answer true if this file really is open."

	^ (self primSizeNoError: sqFileStruct) notNil! !

!IOFileHandle methodsFor: 'converting' stamp: 'dtl 10/28/1999 16:03'!
asSQFileStruct
	"Answer a ByteArray which can be treated as a struct SQFile by the virtual machine.
	This is an optimization for performance, in which the value of the SQFile ByteArray
	is held by my sqFileStruct instance variable."

	"FIXME: Do some testing to see if this really makes any difference. If not, eliminate
	the instance variable and this method."

	^ self sqFileStruct
! !

!IOFileHandle methodsFor: 'updating' stamp: 'dtl 10/29/1999 11:06'!
changed: aParameter

	self update: aParameter.
	^ super changed: aParameter! !

!IOFileHandle methodsFor: 'updating' stamp: 'dtl 10/29/1999 11:00'!
update: aParameter
	"Update the instance variables which depend on sqFileStruct."

	self sqFileStruct ifNotNil: [self file; sessionID; fileSize; writable; lastOp]! !

!IOFileHandle methodsFor: 'printing' stamp: 'dtl 11/20/1999 17:06'!
printOn: aStream

	super printOn: aStream.
	aStream nextPutAll: ' ('.
	aStream nextPutAll: 'sessionID'.
	sessionID do: [ :e | aStream nextPut: Character space. e printOn: aStream].
	writable ifTrue: [aStream nextPutAll: ', writable, ']
			ifFalse: [aStream nextPutAll: ', readonly, '].
	aStream nextPutAll: 'fileSize ', fileSize printString, ', '.
	aStream nextPutAll: 'lastOp ', lastOp.
	aStream nextPutAll: ')'
! !

!IOFileHandle methodsFor: 'private - converting' stamp: 'dtl 10/28/1999 15:48'!
fileFromSqFileStruct

	^ sqFileStruct copyFrom: 1 to: 4! !

!IOFileHandle methodsFor: 'private - converting' stamp: 'dtl 11/1/1999 21:54'!
fileSizeFromSqFileStruct

	^ self integerFromByteArray: (sqFileStruct copyFrom: 13 to: 16)
! !

!IOFileHandle methodsFor: 'private - converting' stamp: 'dtl 11/19/1999 22:03'!
lastOpFromSqFileStruct

	| op |
	op _ self integerFromByteArray: (sqFileStruct copyFrom: 17 to: 20).
	op = 0 ifTrue: [^ #uncommitted].
	op = 1 ifTrue: [^ #read].
	op = 2 ifTrue: [^ #write].
	^ 'undefined op ', op printString! !

!IOFileHandle methodsFor: 'private - converting' stamp: 'dtl 10/28/1999 15:48'!
sessionIDFromSqFileStruct

	^ sqFileStruct copyFrom: 5 to: 8! !

!IOFileHandle methodsFor: 'private - converting' stamp: 'dtl 11/1/1999 21:58'!
writableFromSqFileStruct
	"Answer a boolean, true if writable is not equal to zero"

	^ (self integerFromByteArray: (sqFileStruct copyFrom: 9 to: 12)) ~= 0
! !

!IOFileHandle methodsFor: 'primitives' stamp: 'jm 9/21/1998 16:18'!
primAtEnd: id
	"Answer true if the file position is at the end of the file."

	<primitive: 150>
	self primitiveFailed
! !

!IOFileHandle methodsFor: 'primitives' stamp: 'jm 9/21/1998 14:02'!
primClose: id
	"Close this file."

	<primitive: 151>
	self primitiveFailed
! !

!IOFileHandle methodsFor: 'primitives' stamp: 'jm 9/21/1998 14:02'!
primCloseNoError: id
	"Close this file. Don't raise an error if the primitive fails."

	<primitive: 151>
! !

!IOFileHandle methodsFor: 'primitives' stamp: 'jm 9/21/1998 14:04'!
primGetPosition: id
	"Get this files current position."

	<primitive: 152>
	self primitiveFailed
! !

!IOFileHandle methodsFor: 'primitives' stamp: 'jm 9/21/1998 14:06'!
primOpen: fileName writable: writableFlag
	"Open a file of the given name, and return the file ID obtained.
	If writableFlag is true, then
		if there is none with this name, then create one
		else prepare to overwrite the existing from the beginning
	otherwise
		if the file exists, open it read-only
		else return nil"

	<primitive: 153>
	^ nil
! !

!IOFileHandle methodsFor: 'primitives' stamp: 'jm 9/21/1998 16:18'!
primRead: id into: byteArray startingAt: startIndex count: count
	"Read up to count bytes of data from this file into the given string or byte array starting at the given index. Answer the number of bytes actually read."

	<primitive: 154>
	self closed ifTrue: [^ self error: 'File is closed'].
	self error: 'File read failed'.
! !

!IOFileHandle methodsFor: 'primitives' stamp: 'jm 9/21/1998 14:09'!
primSetPosition: id to: anInteger
	"Set this file to the given position."

	<primitive: 155>
	self primitiveFailed
! !

!IOFileHandle methodsFor: 'primitives' stamp: 'jm 9/21/1998 16:17'!
primSize: id
	"Answer the size of this file."

	<primitive: 157>
	self primitiveFailed
! !

!IOFileHandle methodsFor: 'primitives' stamp: 'jm 9/21/1998 16:17'!
primSizeNoError: id
	"Answer the size of this file. Answer nil if the primitive fails; this indicates that the file handle has become stale."

	<primitive: 157>
	^ nil
! !

!IOFileHandle methodsFor: 'primitives' stamp: 'jm 9/21/1998 16:18'!
primWrite: id from: stringOrByteArray startingAt: startIndex count: count
	"Write count bytes onto this file from the given string or byte array starting at the given index. Answer the number of bytes written."

	<primitive: 158>
	self closed ifTrue: [^ self error: 'File is closed'].
	self error: 'File write failed'.
! !


!IOHandle class reorganize!
('initialize-release' initialize)
('instance creation' forFile:forWrite: newFromSqFileStruct: newFromSqSocketStruct:)
('concrete classes' forFiles forSockets)
('testing' bigEndianMachine isBigEndian isLittleEndian)
('registry' register: registry unregister:)
('constants' sqFileStructureSize sqSocketStructureSize)
!


!IOHandle class methodsFor: 'initialize-release' stamp: 'dtl 11/13/1999 19:00'!
initialize

	BigEndianMachine _ nil.
	Registry _ nil! !

!IOHandle class methodsFor: 'instance creation' stamp: 'dtl 11/13/1999 10:52'!
forFile: fileName forWrite: writeMode

	^ self forFiles new open: fileName forWrite: writeMode
! !

!IOHandle class methodsFor: 'instance creation' stamp: 'dtl 10/23/1999 11:53'!
newFromSqFileStruct: aByteArray

	^ self forFiles newFromSqFileStruct: aByteArray

! !

!IOHandle class methodsFor: 'instance creation' stamp: 'dtl 10/24/1999 23:03'!
newFromSqSocketStruct: aByteArray

	^ self forSockets newFromSqSocketStruct: aByteArray

! !

!IOHandle class methodsFor: 'concrete classes' stamp: 'dtl 10/23/1999 11:43'!
forFiles
	"Answer the subclass which represents file system objects."

	^ IOFileHandle! !

!IOHandle class methodsFor: 'concrete classes' stamp: 'dtl 10/24/1999 23:03'!
forSockets
	"Answer the subclass which represents network sockets."

	^ IOSocketHandle! !

!IOHandle class methodsFor: 'testing' stamp: 'dtl 11/2/1999 05:27'!
bigEndianMachine
	"True if the machine on which we are executing uses big endian byte order conventions.
	Class variable allows a quicker check than #isBigEndian"

	BigEndianMachine ifNil: [BigEndianMachine _ self isBigEndian].
	^ BigEndianMachine! !

!IOHandle class methodsFor: 'testing' stamp: 'dtl 11/2/1999 05:23'!
isBigEndian
	"Answer true if the machine on which we are executing uses big endian byte
	order conventions."

	^ self isLittleEndian not
! !

!IOHandle class methodsFor: 'testing' stamp: 'dtl 11/2/1999 05:22'!
isLittleEndian
	"Answer true if the machine on which we are executing uses little endian byte
	order conventions."

	^ ((ReadStream on: (ByteArray newFrom: #(0 0 4 210))) nextNumber: 4) == 1234! !

!IOHandle class methodsFor: 'registry' stamp: 'dtl 11/13/1999 10:14'!
register: anObject

	WeakArray isFinalizationSupported ifFalse:[^ anObject].
	self registry add: anObject! !

!IOHandle class methodsFor: 'registry' stamp: 'dtl 11/13/1999 10:13'!
registry

	WeakArray isFinalizationSupported ifFalse:[^nil].
	^ Registry isNil
		ifTrue:[Registry := WeakRegistry new]
		ifFalse:[Registry].! !

!IOHandle class methodsFor: 'registry' stamp: 'dtl 11/13/1999 10:14'!
unregister: anObject

	WeakArray isFinalizationSupported ifFalse:[^ anObject].
	self registry remove: anObject ifAbsent:[]! !

!IOHandle class methodsFor: 'constants' stamp: 'dtl 10/27/1999 21:02'!
sqFileStructureSize
	"Answer the size in bytes of an SQFile data structure."

	^20! !

!IOHandle class methodsFor: 'constants' stamp: 'dtl 10/27/1999 21:02'!
sqSocketStructureSize
	"Answer the size in bytes of an SQSocket data structure."

	^12! !


!IOFileHandle class methodsFor: 'instance creation' stamp: 'dtl 11/1/1999 21:59'!
newFromSqFileStruct: aByteArray

	"IOFileHandle newFromSqFileStruct: (ByteArray newFrom: #(1 2 3 4 5 6 7 8 1 0 0 0 117 0 0 0 13 0 0 0))"

	| handle |
	((aByteArray species = ByteArray) and: [ aByteArray size = self sqFileStructureSize ])
		ifFalse: [ self error: 'invalid argument' ]
		ifTrue: [
			handle _ super new sqFileStruct: aByteArray.
			handle file: handle fileFromSqFileStruct.
			handle sessionID: handle sessionIDFromSqFileStruct.
			handle writable: handle writableFromSqFileStruct.
			handle fileSize: handle fileSizeFromSqFileStruct.
			handle lastOp: handle lastOpFromSqFileStruct].
	^ handle! !


!IOSocketHandle commentStamp: 'dtl 10/30/1999 10:38' prior: 0!
I represent an IO channel on a network socket. I keep a ByteArray which corresponds
to a data structure in the Squeak virtual machine. My instance variables are updated
to reflect the current value of this data structure, allowing the data structure to be
indirectly viewed with a normal Smalltalk inspector.!

!IOSocketHandle reorganize!
('initialize-destroy' destroy)
('accessing' privateSocketPtr privateSocketPtr: sessionID socketType socketType: sqSocketStruct sqSocketStruct:)
('socket accessing' connectionStatus localAddress localPort remoteAddress remotePort)
('queries' dataAvailable isConnected isDestroyed isUnconnected isUnconnectedOrInvalid isValid isWaitingForConnection sendDone statusString)
('connection open-close' abortConnection close connectTo:port: listenOn: listenOn:backlogSize:)
('sending-receiving' receiveDataInto:startingAt:count: sendData:startIndex:count:)
('datagrams' setPeer:port: setPort:)
('converting' asSQSocketStruct)
('private - converting' privateSocketPtrFromSqSocketStruct sessionIDFromSqSocketStruct socketTypeFromSqSocketStruct)
('updating' changed: update:)
('printing' printOn:)
('finalization' finalize)
('primitives' primSocket:connectTo:port: primSocket:listenOn: primSocket:listenOn:backlogSize: primSocket:receiveDataInto:startingAt:count: primSocket:sendData:startIndex:count: primSocket:setPort: primSocketAbortConnection: primSocketCloseConnection: primSocketConnectionStatus: primSocketDestroy: primSocketDestroyGently: primSocketError: primSocketLocalAddress: primSocketLocalPort: primSocketReceiveDataAvailable: primSocketRemoteAddress: primSocketRemotePort: primSocketSendDone:)
!


!IOSocketHandle methodsFor: 'initialize-destroy' stamp: 'dtl 11/13/1999 21:02'!
destroy
	"Destroy this socket connection."

	self primSocketDestroy: sqSocketStruct.
	self changed.
	self unregister
! !

!IOSocketHandle methodsFor: 'accessing' stamp: 'dtl 10/29/1999 10:46'!
privateSocketPtr
	"Always update from the current value of sqSocketStruct."

	^ privateSocketPtr _ self privateSocketPtrFromSqSocketStruct! !

!IOSocketHandle methodsFor: 'accessing' stamp: 'dtl 10/28/1999 11:11'!
privateSocketPtr: aByteArray

	privateSocketPtr _ aByteArray! !

!IOSocketHandle methodsFor: 'accessing' stamp: 'dtl 10/29/1999 10:45'!
sessionID
	"Always update from the current value of sqSocketStruct."

	^ sessionID _ self sessionIDFromSqSocketStruct! !

!IOSocketHandle methodsFor: 'accessing' stamp: 'dtl 11/1/1999 22:01'!
socketType
	"Always update from the current value of sqSocketStruct."

	^ socketType _ self socketTypeFromSqSocketStruct! !

!IOSocketHandle methodsFor: 'accessing' stamp: 'dtl 10/28/1999 11:10'!
socketType: aSymbol
	"aSymbol may be #TCP, #UDP etc."

	socketType _ aSymbol! !

!IOSocketHandle methodsFor: 'accessing' stamp: 'dtl 10/28/1999 11:14'!
sqSocketStruct

	^ sqSocketStruct! !

!IOSocketHandle methodsFor: 'accessing' stamp: 'dtl 10/29/1999 11:39'!
sqSocketStruct: aByteArray

	sqSocketStruct _ aByteArray.
	self changed! !

!IOSocketHandle methodsFor: 'socket accessing' stamp: 'dtl 11/14/1999 12:58'!
connectionStatus

	^ self primSocketConnectionStatus: sqSocketStruct.! !

!IOSocketHandle methodsFor: 'socket accessing' stamp: 'dtl 11/13/1999 21:26'!
localAddress

	^ self primSocketLocalAddress: sqSocketStruct
! !

!IOSocketHandle methodsFor: 'socket accessing' stamp: 'dtl 11/13/1999 21:27'!
localPort

	^ self primSocketLocalPort: sqSocketStruct
! !

!IOSocketHandle methodsFor: 'socket accessing' stamp: 'dtl 11/13/1999 21:28'!
remoteAddress

	^ self primSocketRemoteAddress: sqSocketStruct
! !

!IOSocketHandle methodsFor: 'socket accessing' stamp: 'dtl 11/13/1999 21:28'!
remotePort

	^ self primSocketRemotePort: sqSocketStruct
! !

!IOSocketHandle methodsFor: 'queries' stamp: 'dtl 11/13/1999 21:31'!
dataAvailable

	^ self primSocketReceiveDataAvailable: sqSocketStruct
! !

!IOSocketHandle methodsFor: 'queries' stamp: 'dtl 11/14/1999 12:59'!
isConnected
	"Return true if this socket is connected."

	^ self connectionStatus == Connected
! !

!IOSocketHandle methodsFor: 'queries' stamp: 'dtl 11/14/1999 12:44'!
isDestroyed
	"Return true if the socket has been destroyed."

	"If sessionID is non-zero and privateSocketPtr is zero, then the socket has been destroyed."

	^ (((sessionID reject: [ :e | e == 0]) size ~= 0) &
		((privateSocketPtr reject: [ :e | e == 0]) size = 0))


! !

!IOSocketHandle methodsFor: 'queries' stamp: 'dtl 11/14/1999 13:00'!
isUnconnected
	"Return true if this socket's state is Unconnected."

	^ self connectionStatus == Unconnected
! !

!IOSocketHandle methodsFor: 'queries' stamp: 'dtl 11/14/1999 13:05'!
isUnconnectedOrInvalid
	"Return true if this socket is completely disconnected or is invalid."

	| status |
	status _ self connectionStatus.
	^ (status == Unconnected) | (status == InvalidSocket)
! !

!IOSocketHandle methodsFor: 'queries' stamp: 'dtl 11/14/1999 13:01'!
isValid
	"Return true if this socket contains a valid, non-nil socket handle."

	^ self connectionStatus ~= InvalidSocket

! !

!IOSocketHandle methodsFor: 'queries' stamp: 'dtl 11/14/1999 13:01'!
isWaitingForConnection
	"Return true if this socket is waiting for a connection."

	^ self connectionStatus == WaitingForConnection
! !

!IOSocketHandle methodsFor: 'queries' stamp: 'dtl 11/14/1999 12:19'!
sendDone
	"Return true if the most recent send operation on this socket has completed."

	^ self primSocketSendDone: sqSocketStruct
! !

!IOSocketHandle methodsFor: 'queries' stamp: 'dtl 11/14/1999 13:06'!
statusString
	"Return a string describing the status of this socket."

	| status |
	status _ self connectionStatus.
	status == InvalidSocket ifTrue: [^ 'invalidSocketHandle'].
	status == Unconnected ifTrue: [^ 'unconnected'].
	status == WaitingForConnection ifTrue: [^ 'waitingForConnection'].
	status == Connected ifTrue: [^ 'connected'].
	status == OtherEndClosed ifTrue: [^ 'otherEndClosedButNotThisEnd'].
	status == ThisEndClosed ifTrue: [^ 'thisEndClosedButNotOtherEnd'].
	self isDestroyed ifTrue: [^ 'destroyed'].
	^ 'unknown socket status'
! !

!IOSocketHandle methodsFor: 'connection open-close' stamp: 'dtl 11/14/1999 12:54'!
abortConnection
	"Break this connection, no matter what state it is in. Data that has been sent but
	not received will be lost."

	self primSocketAbortConnection: sqSocketStruct.
	self changed! !

!IOSocketHandle methodsFor: 'connection open-close' stamp: 'dtl 11/14/1999 12:49'!
close
	"Close this connection gracefully. For TCP, this sends a close request, but the stream
	remains open until the other side also closes it."

	self primSocketCloseConnection: sqSocketStruct.  "close this end"
	self changed
! !

!IOSocketHandle methodsFor: 'connection open-close' stamp: 'dtl 11/14/1999 13:10'!
connectTo: hostAddress port: port
	"Initiate a connection to the given port at the given host address. This operation will
	return immediately; follow it with waitForConnectionUntil: to wait until the connection
	is established."

	self isUnconnected
		ifFalse: [self error: 'Socket status must be Unconnected before opening a new connection'].
	self primSocket: sqSocketStruct connectTo: hostAddress port: port.
	self changed! !

!IOSocketHandle methodsFor: 'connection open-close' stamp: 'dtl 11/14/1999 13:15'!
listenOn: port
	"Listen for a connection on the given port. This operation will return immediately;
	follow it with waitForConnectionUntil: to wait until a connection is established."

	self isUnconnected
		ifFalse: [self error:
			'Socket status must be Unconnected before listening for a new connection'].
	self primSocket: sqSocketStruct listenOn: port.
	self changed
! !

!IOSocketHandle methodsFor: 'connection open-close' stamp: 'dtl 11/14/1999 13:17'!
listenOn: portNumber backlogSize: backlog
	"Listen for a connection on the given port.
	If this method succeeds, #accept may be used to establish a new connection"

	self isUnconnected
		ifFalse: [self error:
			'Socket status must be Unconnected before listening for a new connection'].
	self primSocket: sqSocketStruct listenOn: portNumber backlogSize: backlog.
	self changed
! !

!IOSocketHandle methodsFor: 'sending-receiving' stamp: 'dtl 11/14/1999 13:25'!
receiveDataInto: buf startingAt: index count: num
	"Receive up to num bytes of data into a buffer starting at the given index. Return
	the number of bytes read or zero if no data is available."

	^ self primSocket: sqSocketStruct
			receiveDataInto: buf
			startingAt: index
			count: num! !

!IOSocketHandle methodsFor: 'sending-receiving' stamp: 'dtl 11/14/1999 13:33'!
sendData: aStringOrByteArray startIndex: index count: num
	"Send data to the remote host through the given socket starting with the given
	byte index of the given byte array. The data sent is 'pushed' immediately.
	Return the number of bytes of data actually sent; any remaining data should
	be re-submitted for sending after the current send operation has completed."

	^self primSocket: sqSocketStruct
			sendData: aStringOrByteArray
			startIndex: index
			count: num! !

!IOSocketHandle methodsFor: 'datagrams' stamp: 'dtl 11/14/1999 20:45'!
setPeer: hostAddress port: port
	"Set the default send/recv address. Similar to connectTo:port:, but bypasses the
	check for isConnected, since a UDP socket appears connected always."

	self primSocket: sqSocketStruct connectTo: hostAddress port: port.
	self changed! !

!IOSocketHandle methodsFor: 'datagrams' stamp: 'dtl 11/14/1999 15:49'!
setPort: port
	"Associate a local port number with a UDP socket.  Not applicable to TCP sockets."

	self primSocket: sqSocketStruct setPort: port.
	self changed
! !

!IOSocketHandle methodsFor: 'converting' stamp: 'dtl 10/29/1999 10:41'!
asSQSocketStruct

	"Answer a ByteArray which can be treated as a struct SQSocket by the virtual machine.
	This is an optimization for performance, in which the value of the SQSocket ByteArray
	is held by my sqSocketStruct instance variable."

	"FIXME: Do some testing to see if this really makes any difference. If not, eliminate
	the instance variable and this method."

	^ self sqSocketStruct



! !

!IOSocketHandle methodsFor: 'private - converting' stamp: 'dtl 10/28/1999 19:27'!
privateSocketPtrFromSqSocketStruct

	^ sqSocketStruct copyFrom: 9 to: 12! !

!IOSocketHandle methodsFor: 'private - converting' stamp: 'dtl 10/28/1999 19:26'!
sessionIDFromSqSocketStruct

	^ sqSocketStruct copyFrom: 1 to: 4! !

!IOSocketHandle methodsFor: 'private - converting' stamp: 'dtl 11/1/1999 22:00'!
socketTypeFromSqSocketStruct

	^ self class socketTypeFromInteger: (self integerFromByteArray: (sqSocketStruct copyFrom: 5 to: 8))! !

!IOSocketHandle methodsFor: 'updating' stamp: 'dtl 10/29/1999 11:08'!
changed: aParameter

	self update: aParameter.
	^ super changed: aParameter! !

!IOSocketHandle methodsFor: 'updating' stamp: 'dtl 10/29/1999 11:01'!
update: aParameter
	"Update the instance variables which depend on sqSocketStruct."

	self sqSocketStruct ifNotNil: [self sessionID; socketType; privateSocketPtr]! !

!IOSocketHandle methodsFor: 'printing' stamp: 'dtl 11/20/1999 17:08'!
printOn: aStream

	super printOn: aStream.
	socketType ifNotNil: [aStream nextPutAll: (' for ', socketType asString)].
	aStream nextPutAll: ' (sessionID'.
	sessionID do: [ :e | aStream nextPut: Character space. e printOn: aStream].
	aStream nextPutAll: ', privateSocketPtr'.
	privateSocketPtr do: [ :e | aStream nextPut: Character space. e printOn: aStream].



"	aStream nextPutAll: ' ('.
	self sqSocketStruct inject: true into: [
		:first :e | 
		first ifFalse: [ aStream nextPut: Character space ].
		e printOn: aStream.
		false ]."
	aStream nextPutAll: ')'

! !

!IOSocketHandle methodsFor: 'finalization' stamp: 'dtl 11/14/1999 16:02'!
finalize

	self primSocketDestroyGently: sqSocketStruct.
	self changed
! !

!IOSocketHandle methodsFor: 'primitives'!
primSocket: IOSocketHandleID connectTo: hostAddress port: port
	"Attempt to establish a connection to the given port of the given host. This is an asynchronous call; query the IOSocketHandle status to discover if and when the connection is actually completed."

	<primitive: 217>
	self primitiveFailed
! !

!IOSocketHandle methodsFor: 'primitives'!
primSocket: IOSocketHandleID listenOn: port
	"Listen for a connection on the given port. This is an asynchronous call; query the IOSocketHandle status to discover if and when the connection is actually completed."

	<primitive: 218>
	self primitiveFailed
! !

!IOSocketHandle methodsFor: 'primitives' stamp: 'ar 7/16/1999 18:25'!
primSocket: aHandle listenOn: portNumber backlogSize: backlog
	"Primitive. Set up the IOSocketHandle to listen on the given port.
	Will be used in conjunction with #accept only."
	<primitive: 218>
	self destroy. "Accept not supported so clean up"! !

!IOSocketHandle methodsFor: 'primitives'!
primSocket: IOSocketHandleID receiveDataInto: aStringOrByteArray startingAt: startIndex count: count
	"Receive data from the given IOSocketHandle into the given array starting at the given index. Return the number of bytes read or zero if no data is available."

	<primitive: 221>
	self primitiveFailed
! !

!IOSocketHandle methodsFor: 'primitives'!
primSocket: IOSocketHandleID sendData: aStringOrByteArray startIndex: startIndex count: count
	"Send data to the remote host through the given IOSocketHandle starting with the given byte index of the given byte array. The data sent is 'pushed' immediately. Return the number of bytes of data actually sent; any remaining data should be re-submitted for sending after the current send operation has completed."
	"Note: In general, it many take several sendData calls to transmit a large data array since the data is sent in send-buffer-sized chunks. The size of the send buffer is determined when the IOSocketHandle is created."

	<primitive: 223>
	self primitiveFailed
! !

!IOSocketHandle methodsFor: 'primitives' stamp: 'ar 4/30/1999 04:31'!
primSocket: IOSocketHandleID setPort: port
	"Set the local port associated with a UDP IOSocketHandle.
	Note: this primitive is overloaded.  The primitive will not fail on a TCP IOSocketHandle, but
	the effects will not be what was desired.  Best solution would be to split IOSocketHandle into
	two subclasses, TCPIOSocketHandle and UDPIOSocketHandle."

	<primitive: 218>
	self primitiveFailed
! !

!IOSocketHandle methodsFor: 'primitives'!
primSocketAbortConnection: IOSocketHandleID
	"Terminate the connection on the given port immediately without going through the normal close sequence. This is an asynchronous call; query the IOSocketHandle status to discover if and when the connection is actually terminated."

	<primitive: 220>
	self primitiveFailed
! !

!IOSocketHandle methodsFor: 'primitives'!
primSocketCloseConnection: IOSocketHandleID
	"Close the connection on the given port. The remote end is informed that this end has closed and will do no further sends. This is an asynchronous call; query the IOSocketHandle status to discover if and when the connection is actually closed."

	<primitive: 219>
	self primitiveFailed
! !

!IOSocketHandle methodsFor: 'primitives' stamp: 'jm 9/17/97 14:45'!
primSocketConnectionStatus: IOSocketHandleID
	"Return an integer reflecting the connection status of this IOSocketHandle. For a list of possible values, see the comment in the 'initialize' method of this class. If the primitive fails, return a status indicating that the IOSocketHandle handle is no longer valid, perhaps because the Squeak image was saved and restored since the IOSocketHandle was created. (IOSocketHandles do not survive snapshots.)"

	<primitive: 211>
	^ InvalidIOSocketHandle
! !

!IOSocketHandle methodsFor: 'primitives'!
primSocketDestroy: IOSocketHandleID
	"Release the resources associated with this IOSocketHandle. If a connection is open, it is aborted."

	<primitive: 210>
	self primitiveFailed
! !

!IOSocketHandle methodsFor: 'primitives' stamp: 'ar 3/21/98 17:43'!
primSocketDestroyGently: IOSocketHandleID
	"Release the resources associated with this IOSocketHandle. If a connection is open, it is aborted.
	Do not fail if the receiver is already closed."

	<primitive: 210>
! !

!IOSocketHandle methodsFor: 'primitives'!
primSocketError: IOSocketHandleID
	"Return an integer encoding the most recent error on this IOSocketHandle. Zero means no error."

	<primitive: 212>
	self primitiveFailed
! !

!IOSocketHandle methodsFor: 'primitives'!
primSocketLocalAddress: IOSocketHandleID
	"Return the local host address for this IOSocketHandle."

	<primitive: 213>
	self primitiveFailed
! !

!IOSocketHandle methodsFor: 'primitives'!
primSocketLocalPort: IOSocketHandleID
	"Return the local port for this IOSocketHandle, or zero if no port has yet been assigned."

	<primitive: 214>
	self primitiveFailed
! !

!IOSocketHandle methodsFor: 'primitives' stamp: 'jm 9/11/97 20:22'!
primSocketReceiveDataAvailable: IOSocketHandleID
	"Return true if data may be available for reading from the current IOSocketHandle."

	<primitive: 222>
	self primitiveFailed
! !

!IOSocketHandle methodsFor: 'primitives'!
primSocketRemoteAddress: IOSocketHandleID
	"Return the remote host address for this IOSocketHandle, or zero if no connection has been made."

	<primitive: 215>
	self primitiveFailed
! !

!IOSocketHandle methodsFor: 'primitives'!
primSocketRemotePort: IOSocketHandleID
	"Return the remote port for this IOSocketHandle, or zero if no connection has been made."

	<primitive: 216>
	self primitiveFailed
! !

!IOSocketHandle methodsFor: 'primitives'!
primSocketSendDone: IOSocketHandleID
	"Return true if there is no send in progress on the current IOSocketHandle."

	<primitive: 224>
	self primitiveFailed
! !


!IOSocketHandle class methodsFor: 'class initialization' stamp: 'dtl 11/13/1999 21:38'!
initialize
	"IOSocketHandle initialize"

	"Socket Status Values"
	InvalidSocket _ -1.
	Unconnected _ 0.
	WaitingForConnection _ 1.
	Connected _ 2.
	OtherEndClosed _ 3.
	ThisEndClosed _ 4.
! !

!IOSocketHandle class methodsFor: 'instance creation' stamp: 'dtl 11/13/1999 21:17'!
acceptFrom: aSocketHandle receiveBufferSize: rbSize sendBufSize: sbSize semaIndex: index
	"Create a new socket handle from an accept call on another socket handle."

	| sqSocketStruct |
	sqSocketStruct _ self primAcceptFrom: aSocketHandle sqSocketStruct
						receiveBufferSize: rbSize
						sendBufSize: sbSize
						semaIndex: index.
	sqSocketStruct ifNil: [^ nil].
	^ self newFromSqSocketStruct: sqSocketStruct

! !

!IOSocketHandle class methodsFor: 'instance creation' stamp: 'dtl 11/13/1999 21:16'!
createNetwork: netType type: socketType receiveBufferSize: rcvBufSize sendBufSize: sendBufSize semaIndex: semaIndex

	| sqSocketStruct |

	sqSocketStruct _ self primSocketCreateNetwork: netType
							type: socketType
							receiveBufferSize: rcvBufSize
							sendBufSize: sendBufSize
							semaIndex: semaIndex.
	sqSocketStruct ifNil: [^ nil].
	^ self newFromSqSocketStruct: sqSocketStruct
! !

!IOSocketHandle class methodsFor: 'instance creation' stamp: 'dtl 11/1/1999 22:01'!
newFromSqSocketStruct: aByteArray

	"IOSocketHandle newFromSqSocketStruct: (ByteArray newFrom: #(1 2 3 4 0 0 0 0 9 10 11 12))"

	| handle |
	((aByteArray species = ByteArray) and: [ aByteArray size = self sqSocketStructureSize ])
		ifFalse: [ self error: 'invalid argument' ]
		ifTrue: [
			handle _ super new sqSocketStruct: aByteArray.
			handle sessionID: handle sessionIDFromSqSocketStruct.
			handle socketType: handle socketTypeFromSqSocketStruct.
			handle privateSocketPtr: handle privateSocketPtrFromSqSocketStruct].
	^ handle! !

!IOSocketHandle class methodsFor: 'private - converting' stamp: 'dtl 10/28/1999 11:57'!
socketTypeAsInteger: aSocketTypeSymbol
	"FIXME: The mapping of socket types to integers should be in class Socket. Socket has
	class variables for this, but no accessors. Somebody please gather this into one class,
	then update or elimitate this method."

	(aSocketTypeSymbol == #TCP) ifTrue: [^ 0].
	(aSocketTypeSymbol == #UDP) ifTrue: [^ 1].
	self error: 'invalid socket type, should be a symbol such as #TCP'! !

!IOSocketHandle class methodsFor: 'private - converting' stamp: 'dtl 10/28/1999 11:58'!
socketTypeFromInteger: anInteger
	"FIXME: The mapping of socket types to integers should be in class Socket. Socket has
	class variables for this, but no accessors. Somebody please gather this into one class,
	then update or elimitate this method."

	(anInteger == 0) ifTrue: [^ #TCP].
	(anInteger == 1) ifTrue: [^ #UDP].
	self error: 'invalid socket type, should be 0 or 1'! !

!IOSocketHandle class methodsFor: 'primitives' stamp: 'dtl 11/13/1999 20:49'!
primAcceptFrom: aHandle receiveBufferSize: rcvBufSize sendBufSize: sndBufSize semaIndex: semaIndex
	"Create and return a new IOSocketHandle handle based on accepting the connection from the given listening IOSocketHandle"
	<primitive: 225>
	^self primitiveFailed! !

!IOSocketHandle class methodsFor: 'primitives' stamp: 'dtl 11/13/1999 21:13'!
primSocketCreateNetwork: netType type: socketType receiveBufferSize: rcvBufSize sendBufSize: sendBufSize semaIndex: semaIndex
	"Return a new socket handle for a socket of the given type and buffer sizes. Return nil if socket creation fails.
	The netType parameter is platform dependent and can be used to encode both the protocol type (IP, Xerox XNS, etc.) and/or the physical network interface to use if this host is connected to multiple networks. A zero netType means to use IP protocols and the primary (or only) network interface.
	The socketType parameter specifies:
		0	reliable stream socket (TCP if the protocol is IP)
		1	unreliable datagram socket (UDP if the protocol is IP)
	The buffer size parameters allow performance to be tuned to the application. For example, a larger receive buffer should be used when the application expects to be receiving large amounts of data, especially from a host that is far away. These values are considered requests only; the underlying implementation will ensure that the buffer sizes actually used are within allowable bounds. Note that memory may be limited, so an application that keeps many sockets open should use smaller buffer sizes.
 	If semaIndex is > 0, it is taken to be the index of a Semaphore in the external objects array to be associated with this socket. This semaphore will be signalled when the socket status changes, such as when data arrives or a send completes. All processes waiting on the semaphore will be awoken for each such event; each process must then query the socket state to figure out if the conditions they are waiting for have been met. For example, a process waiting to send some data can see if the last send has completed."

	<primitive: 209>
	^ nil  "socket creation failed"
! !


!Socket commentStamp: 'dtl 10/30/1999 11:22' prior: 0!
A Socket represents a network connection point. Current sockets are designed to support the TCP/IP and UDP protocols, although UDP is not yet implemented. It should be possible to support other protocols (such as AppleTalk) sockets by extending the socket primitives.

Subclasses of socket provide support for network protocols such as POP, NNTP, HTTP, and FTP. Sockets also allow you to implement your own custom services and may be used to support Remote Procedure Call or Remote Method Invocation some day.

Note: Now uses an IOSocketHandle. The #changed message is sent to ioSocketHandle when the connection status changes, but not when reading and writing occurs. There is no actual change to the SQSocket data structure in most cases, so the #changed message is there in anticipation of moving more responsibilities into the IOSocketHandle class. Alternatively, it may make more sense to eventually refactor such that a Socket is itself a kind of IOHandle, and the streaming types of behavior move into the Stream hierarchy. That's a bigger issue, so I won't worry about it now. - dtl Oct 30, 1999.
!
]style[(347 4 6 3 745)f1,f1LHTTPSocket Comment;,f1,f1LFTPSocket Comment;,f1!

!Socket methodsFor: 'initialize-destroy' stamp: 'dtl 11/13/1999 20:55'!
acceptFrom: aSocket
	"Initialize a new socket handle from an accept call"

	| semaIndex |
	semaphore _ Semaphore new.
	semaIndex _ Smalltalk registerExternalObject: semaphore.
	self ioSocketHandle: (IOHandle forSockets acceptFrom: aSocket ioSocketHandle
											receiveBufferSize: 8000
											sendBufSize: 8000
											semaIndex: semaIndex).
	ioSocketHandle = nil ifTrue: [  "socket creation failed"
		Smalltalk unregisterExternalObject: semaphore.
		semaphore _ nil
	] ifFalse: [self register].
! !

!Socket methodsFor: 'initialize-destroy' stamp: 'dtl 11/13/1999 21:03'!
destroy
	"Destroy this socket. Its connection, if any, is aborted and its resources are freed. Do nothing if the socket has already been destroyed (i.e., if its ioSocketHandle is nil)."

	ioSocketHandle = nil
		ifFalse: [
			self isValid ifTrue: [ioSocketHandle destroy].
			Smalltalk unregisterExternalObject: semaphore.
			ioSocketHandle _ nil.
			semaphore _ nil.
			self unregister].
! !

!Socket methodsFor: 'initialize-destroy' stamp: 'dtl 11/13/1999 21:22'!
initialize: socketType
	"Initialize a new socket handle. If socket creation fails, ioSocketHandle will be set to nil."

	| semaIndex |
	semaphore _ Semaphore new.
	semaIndex _ Smalltalk registerExternalObject: semaphore.
	self ioSocketHandle: (IOHandle forSockets createNetwork: 0
										type: socketType
										receiveBufferSize: 8000
										sendBufSize: 8000
										semaIndex: semaIndex).
	ioSocketHandle = nil ifTrue: [  "socket creation failed"
		Smalltalk unregisterExternalObject: semaphore.
		semaphore _ nil
	] ifFalse: [self register].
! !

!Socket methodsFor: 'accessing' stamp: 'dtl 10/30/1999 10:46'!
ioSocketHandle

	^ ioSocketHandle! !

!Socket methodsFor: 'accessing' stamp: 'dtl 10/30/1999 10:47'!
ioSocketHandle: anIOHandle

	ioSocketHandle _ anIOHandle! !

!Socket methodsFor: 'accessing' stamp: 'dtl 11/13/1999 21:26'!
localAddress

	^ ioSocketHandle localAddress

! !

!Socket methodsFor: 'accessing' stamp: 'dtl 11/13/1999 21:27'!
localPort

	^ ioSocketHandle localPort

! !

!Socket methodsFor: 'accessing' stamp: 'dtl 11/13/1999 21:28'!
remoteAddress

	^ ioSocketHandle remoteAddress

! !

!Socket methodsFor: 'accessing' stamp: 'dtl 11/13/1999 21:29'!
remotePort

	^ ioSocketHandle remotePort

! !

!Socket methodsFor: 'queries' stamp: 'dtl 11/13/1999 21:31'!
dataAvailable
	"Return true if this socket has unread received data."

	ioSocketHandle == nil ifTrue: [^ false].
	^ ioSocketHandle dataAvailable

! !

!Socket methodsFor: 'queries' stamp: 'dtl 11/13/1999 21:40'!
isConnected
	"Return true if this socket is connected."

	ioSocketHandle == nil ifTrue: [^ false].
	^ ioSocketHandle isConnected

! !

!Socket methodsFor: 'queries' stamp: 'dtl 11/13/1999 22:46'!
isUnconnected
	"Return true if this socket's state is Unconnected."

	ioSocketHandle == nil ifTrue: [^ false].
	^ ioSocketHandle isUnconnected

! !

!Socket methodsFor: 'queries' stamp: 'dtl 11/13/1999 22:47'!
isUnconnectedOrInvalid
	"Return true if this socket is completely disconnected or is invalid."

	ioSocketHandle == nil ifTrue: [^ true].
	^ ioSocketHandle isUnconnectedOrInvalid

! !

!Socket methodsFor: 'queries' stamp: 'dtl 11/14/1999 12:17'!
isValid
	"Return true if this socket contains a valid, non-nil socket handle."

	ioSocketHandle == nil ifTrue: [^ false].
	^ ioSocketHandle isValid

! !

!Socket methodsFor: 'queries' stamp: 'dtl 11/14/1999 12:17'!
isWaitingForConnection
	"Return true if this socket is waiting for a connection."

	ioSocketHandle == nil ifTrue: [^ false].
	^ ioSocketHandle isWaitingForConnection

! !

!Socket methodsFor: 'queries' stamp: 'dtl 11/14/1999 12:19'!
sendDone
	"Return true if the most recent send operation on this socket has completed."

	ioSocketHandle == nil ifTrue: [^ false].
	^ ioSocketHandle sendDone
! !

!Socket methodsFor: 'queries' stamp: 'dtl 11/14/1999 12:47'!
statusString
	"Return a string describing the status of this socket."

	ioSocketHandle == nil
		ifTrue: [^ 'destroyed']
		ifFalse: [^ ioSocketHandle statusString]

! !

!Socket methodsFor: 'connection open/close' stamp: 'dtl 10/30/1999 10:57'!
accept
	"Accept a connection from the receiver socket.
	Return a new socket that is connected to the client"

	^ Socket acceptFrom: self.! !

!Socket methodsFor: 'connection open/close' stamp: 'dtl 11/14/1999 12:50'!
close
	"Close this connection gracefully. For TCP, this sends a close request, but the stream remains open until the other side also closes it."

	^ ioSocketHandle close

! !

!Socket methodsFor: 'connection open/close' stamp: 'dtl 10/30/1999 11:34'!
closeAndDestroy
	"First, try to close this connection gracefully. If the close attempt fails or times out, abort the connection. In either case, destroy the socket. Do nothing if the socket has already been destroyed (i.e., if its ioSocketHandle is nil)."

	self closeAndDestroy: 20.

! !

!Socket methodsFor: 'connection open/close' stamp: 'dtl 11/14/1999 12:54'!
closeAndDestroy: timeoutSeconds
	"First, try to close this connection gracefully. If the close attempt fails or times out, abort the connection. In either case, destroy the socket. Do nothing if the socket has already been destroyed (i.e., if its ioSocketHandle is nil)."

	ioSocketHandle = nil
		ifFalse: [
			self isConnected ifTrue: [
				self close.  "close this end"
				(self waitForDisconnectionUntil: (Socket deadlineSecs: timeoutSeconds))
					ifFalse: [
						"if the other end doesn't close soon, just abort the connection"
						ioSocketHandle abortConnection]].
			self destroy].
! !

!Socket methodsFor: 'connection open/close' stamp: 'dtl 11/14/1999 13:11'!
connectTo: hostAddress port: port
	"Initiate a connection to the given port at the given host address. This operation will
	return immediately; follow it with waitForConnectionUntil: to wait until the connection
	is established."

	ioSocketHandle connectTo: hostAddress port: port

! !

!Socket methodsFor: 'connection open/close' stamp: 'dtl 11/14/1999 12:55'!
disconnect
	"Break this connection, no matter what state it is in. Data that has been sent but not received will be lost."

	ioSocketHandle ifNotNil: [ioSocketHandle abortConnection]

! !

!Socket methodsFor: 'connection open/close' stamp: 'dtl 11/14/1999 13:19'!
listenOn: portNumber
	"Listen for a connection on the given port. This operation will return immediately;
	follow it with waitForConnectionUntil: to wait until a connection is established."

	ioSocketHandle ifNotNil: [ioSocketHandle listenOn: portNumber]

! !

!Socket methodsFor: 'connection open/close' stamp: 'dtl 11/14/1999 13:18'!
listenOn: portNumber backlogSize: backlog
	"Listen for a connection on the given port.
	If this method succeeds, #accept may be used to establish a new connection"

	ioSocketHandle ifNotNil: [ioSocketHandle listenOn: portNumber backlogSize: backlog]

! !

!Socket methodsFor: 'sending-receiving' stamp: 'dtl 11/14/1999 13:26'!
getData
	"Get some data"

	| buf bytesRead |
	(self waitForDataUntil: Socket standardDeadline) ifFalse: [self error: 'getData timeout'].
	buf _ String new: 2000.
	bytesRead _ ioSocketHandle receiveDataInto: buf startingAt: 1 count: buf size.
	^ buf copyFrom: 1 to: bytesRead! !

!Socket methodsFor: 'sending-receiving' stamp: 'dtl 11/14/1999 13:30'!
readInto: aStringOrByteArray startingAt: aNumber
	"Read data into the given buffer starting at the given index and return the number of bytes
	received. Note the given buffer may be only partially filled by the received data."

	(self waitForDataUntil: Socket standardDeadline) ifFalse: [self error: 'receive timeout'].
	^ ioSocketHandle receiveDataInto: aStringOrByteArray
						startingAt: aNumber
						count: aStringOrByteArray size - aNumber + 1

! !

!Socket methodsFor: 'sending-receiving' stamp: 'dtl 11/14/1999 13:30'!
receiveDataInto: aStringOrByteArray
	"Receive data into the given buffer and return the number of bytes received. Note the
	given buffer may be only partially filled by the received data."

	^ ioSocketHandle receiveDataInto: aStringOrByteArray
						startingAt: 1
						count: aStringOrByteArray size

! !

!Socket methodsFor: 'sending-receiving' stamp: 'dtl 11/14/1999 13:34'!
sendData: aStringOrByteArray
	"Send all of the data in the given array, even if it requires multiple calls to send it all.
	Return the number of bytes sent."

	| bytesSent bytesToSend count |
	bytesToSend _ aStringOrByteArray size.
	bytesSent _ 0.
	[bytesSent < bytesToSend] whileTrue: [
		(self waitForSendDoneUntil: (Socket deadlineSecs: 20))
			ifFalse: [self error: 'send data timeout; data not sent'].
		count _ ioSocketHandle sendData: aStringOrByteArray
								startIndex: bytesSent + 1
								count: bytesToSend - bytesSent.
		bytesSent _ bytesSent + count].

	^ bytesSent
! !

!Socket methodsFor: 'sending-receiving' stamp: 'dtl 11/14/1999 13:35'!
sendSomeData: aStringOrByteArray startIndex: startIndex count: count
	"Send up to count bytes of the given data starting at the given index. Answer the number of bytes actually sent."
	"Note: This operation may have to be repeated multiple times to send a large amount of data."

	| bytesSent |
	(self waitForSendDoneUntil: (Socket deadlineSecs: 20))
		ifTrue: [
			bytesSent _ ioSocketHandle sendData: aStringOrByteArray
										startIndex: startIndex
										count: count]
		ifFalse: [self error: 'send data timeout; data not sent'].
	^ bytesSent
! !

!Socket methodsFor: 'waiting' stamp: 'dtl 11/14/1999 13:41'!
waitForConnectionUntil: deadline
	"Wait up until the given deadline for a connection to be established. Return true if it is established by the deadline, false if not."

	[(self isWaitingForConnection) and: [Time millisecondClockValue < deadline]]
		whileTrue: [semaphore waitTimeoutMSecs: (deadline - Time millisecondClockValue)].
	ioSocketHandle changed.
	^ Connected
! !

!Socket methodsFor: 'waiting' stamp: 'dtl 11/14/1999 13:44'!
waitForDataUntil: deadline
	"Wait up until the given deadline for data to arrive. Return true if data arrives by the deadline, false if not."

	| dataArrived |
	[self isConnected & 
	 (dataArrived _ self dataAvailable) not
			"Connection end and final data can happen fast, so test in this order"
		and: [Time millisecondClockValue < deadline]] whileTrue: [
			semaphore waitTimeoutMSecs: (deadline - Time millisecondClockValue)].

	^ dataArrived
! !

!Socket methodsFor: 'waiting' stamp: 'dtl 11/14/1999 13:50'!
waitForDisconnectionUntil: deadline
	"Wait up until the given deadline for the the connection to be broken. Return true if it
	is broken by the deadline, false if not."
	"Note: The client should know the the connect is really going to be closed (e.g., because
	he has called 'close' to send a close request to the other end) before calling this method."

	| extraBytes status |
	extraBytes _ 0.
	status _ ioSocketHandle connectionStatus.
	[((status = Connected) or: [status = ThisEndClosed]) and:
	 [Time millisecondClockValue < deadline]] whileTrue: [
		self dataAvailable
			ifTrue: [extraBytes _ extraBytes + self discardReceivedData].
		semaphore waitTimeoutMSecs: (deadline - Time millisecondClockValue).
		status _ ioSocketHandle connectionStatus].
	extraBytes > 0
		ifTrue: [self inform: 'Discarded ', extraBytes printString, ' bytes while closing connection.'].
	ioSocketHandle changed.
	^ status ~= Connected
! !

!Socket methodsFor: 'waiting' stamp: 'dtl 11/14/1999 13:52'!
waitForSendDoneUntil: deadline
	"Wait up until the given deadline for the current send operation to complete. Return true if it completes by the deadline, false if not."

	| sendDone |
	[self isConnected & (sendDone _ self sendDone) not
			"Connection end and final data can happen fast, so test in this order"
		and: [Time millisecondClockValue < deadline]] whileTrue: [
			semaphore waitTimeoutMSecs: (deadline - Time millisecondClockValue)].
	^ sendDone! !

!Socket methodsFor: 'finalization' stamp: 'dtl 11/14/1999 16:02'!
finalize

	ioSocketHandle finalize.
	Smalltalk unregisterExternalObject: semaphore.
! !

!Socket methodsFor: 'datagrams' stamp: 'dtl 11/14/1999 20:46'!
setPeer: hostAddress port: port
	"Set the default send/recv address."

	ioSocketHandle setPeer: hostAddress port: port

! !

!Socket methodsFor: 'datagrams' stamp: 'dtl 11/14/1999 15:50'!
setPort: port
	"Associate a local port number with a UDP socket.  Not applicable to TCP sockets."

	ioSocketHandle setPort: port

! !

!Socket methodsFor: 'other' stamp: 'dtl 11/14/1999 15:51'!
getResponseNoLF
	"Get the response to the last command."

	| buf response bytesRead c lf |
	(self waitForDataUntil: (Socket deadlineSecs: 20)) ifFalse: [
		self error: 'getResponse timeout'].
	lf _ Character lf.
	buf _ String new: 1000.
	response _ WriteStream on: ''.
	[self dataAvailable] whileTrue: [
		bytesRead _ ioSocketHandle receiveDataInto: buf
									startingAt: 1
									count: buf size.
		1 to: bytesRead do: [ :i |
			(c _ buf at: i) ~= lf ifTrue: [response nextPut: c]]].

	^ response contents
! !


!HTTPSocket methodsFor: 'as yet unclassified' stamp: 'dtl 11/14/1999 15:57'!
getResponseUpTo: markerString
	"Keep reading until the marker is seen.  Return three parts: header, marker, beginningOfData.  Fails if no marker in first 2000 chars." 

	| buf response bytesRead tester mm |
	buf _ String new: 2000.
	response _ WriteStream on: buf.
	tester _ 1. mm _ 1.
	[tester _ tester - markerString size + 1 max: 1.  "rewind a little, in case the marker crosses a read boundary"
	tester to: response position do: [:tt |
		(buf at: tt) = (markerString at: mm) ifTrue: [mm _ mm + 1] ifFalse: [mm _ 1].
			"Not totally correct for markers like xx0xx"
		mm > markerString size ifTrue: ["got it"
			^ Array with: (buf copyFrom: 1 to: tt+1-mm)
				with: markerString
				with: (buf copyFrom: tt+1 to: response position)]].
	 tester _ 1 max: response position.	"OK if mm in the middle"
	 (response position < buf size) & (self isConnected | self dataAvailable)] whileTrue: [
		(self waitForDataUntil: (Socket deadlineSecs: 5)) ifFalse: [
			Transcript show: 'data was late'; cr].
		bytesRead _ ioSocketHandle receiveDataInto: buf 
			startingAt: response position + 1 count: buf size - response position.
		"response position+1 to: response position+bytesRead do: [:ii | 
			response nextPut: (buf at: ii)].	totally redundant, but needed to advance position!!"
		response instVarAt: 2 "position" put: 
			(response position + bytesRead)].	"horrible, but fast"

	^ Array with: response contents
		with: ''
		with: ''		"Marker not found and connection closed"
! !

!HTTPSocket methodsFor: 'as yet unclassified' stamp: 'dtl 11/14/1999 15:57'!
getResponseUpTo: markerString ignoring: ignoreString
	"Keep reading, until the marker is seen, skipping characters in ignoreString when
      comparing to the marker.  Return three parts: header, marker, beginningOfData.
     Fails if no marker in first 2000 chars." 

	| buf response bytesRead tester mm skipped |
	buf _ String new: 2000.
	response _ WriteStream on: buf.
	tester _ 1. mm _ 1.
	skipped _ 0.
	[tester _ tester - markerString size + 1 max: 1.  "rewind a little, in case the marker crosses a read boundary"
	tester to: response position do: [:tt |
		(buf at: tt) = (markerString at: mm) ifFalse:
			[[ignoreString includes: (markerString at: mm)] whileTrue:
				[mm _ mm + 1. skipped _ skipped + 1]].
		(buf at: tt) = (markerString at: mm)
			ifTrue: [mm _ mm + 1]
			ifFalse: [mm _ 1. skipped _ 0].
			"Not totally correct for markers like xx0xx"
		mm > markerString size ifTrue: ["got it"
			^ Array with: (buf copyFrom: 1 to: tt+1-mm+skipped)
				with: markerString
				with: (buf copyFrom: tt+1 to: response position)]].
	 tester _ 1 max: response position.	"OK if mm in the middle"
	 (response position < buf size) & (self isConnected | self dataAvailable)] whileTrue: [
		(self waitForDataUntil: (Socket deadlineSecs: 5)) ifFalse: [
			Transcript show: 'data was late'; cr].
		bytesRead _ ioSocketHandle receiveDataInto: buf 
			startingAt: response position + 1 count: buf size - response position.
		"response position+1 to: response position+bytesRead do: [:ii | 
			response nextPut: (buf at: ii)].	totally redundant, but needed to advance position!!"
		response instVarAt: 2 "position" put: 
			(response position + bytesRead)].	"horrible, but fast"

	^ Array with: response contents
		with: ''
		with: ''		"Marker not found and connection closed"
! !

!HTTPSocket methodsFor: 'as yet unclassified' stamp: 'dtl 11/14/1999 15:57'!
getRestOfBuffer: beginning
	"We don't know the length.  Keep going until connection is closed.  Part of it has already been received.  Response is of type text, not binary."

	| buf response bytesRead |
	response _ RWBinaryOrTextStream on: (String new: 2000).
	response nextPutAll: beginning.
	buf _ String new: 2000.

	[self isConnected | self dataAvailable] 
	whileTrue: [
		(self waitForDataUntil: (Socket deadlineSecs: 5)) ifTrue: [
	 		Transcript show: 'data was slow'; cr].
		bytesRead _ ioSocketHandle receiveDataInto: buf 
				startingAt: 1 count: buf size. 
		bytesRead > 0 ifTrue: [  
			response nextPutAll: (buf copyFrom: 1 to: bytesRead)] ].
	Transcript cr; show: 'data byte count: ', response position printString.
	response reset.	"position: 0."
	^ response
! !

!HTTPSocket methodsFor: 'as yet unclassified' stamp: 'dtl 11/14/1999 15:57'!
getRestOfBuffer: beginning totalLength: length
	"Reel in a string of a fixed length.  Part of it has already been received.  Close the connection after all chars are received.  We do not strip out linefeed chars.  tk 6/16/97 22:32" 
	"if length is nil, read until connection close.  Response is of type text, not binary."

	| buf response bytesRead |
	length ifNil: [^ self getRestOfBuffer: beginning].
	buf _ String new: length.
	response _ RWBinaryOrTextStream on: buf.
	response nextPutAll: beginning.
	buf _ String new: length.

	[(response position < length) & (self isConnected | self dataAvailable)] 
	whileTrue: [
		(self waitForDataUntil: (Socket deadlineSecs: 5)) ifFalse: [
	 		Transcript show: 'data was slow'; cr].
		bytesRead _ ioSocketHandle receiveDataInto: buf startingAt: 1 
				count: (length - response size). 
		bytesRead > 0 ifTrue: [  
			response nextPutAll: (buf copyFrom: 1 to: bytesRead)] ].
	"Transcript cr; show: 'data byte count: ', response position printString."
	"Transcript cr; show: ((self isConnected) ifTrue: ['Over length by: ', bytesRead printString] 
		ifFalse: ['Socket closed'])."
	response position < length ifTrue: [^ 'server aborted early'].
	response reset.	"position: 0."
	^ response
! !


!FTPSocket methodsFor: 'as yet unclassified' stamp: 'dtl 11/14/1999 15:56'!
getAllData
	"Reel in all data until the server closes the connection.  Return a RWBinaryOrTextStream.  Don't know how much is coming."

	| buf response bytesRead |
	buf _ String new: 4000.
	response _ RWBinaryOrTextStream on: (String new: 4000).
	[(self dataAvailable | self isConnected)] whileTrue: [
		(self waitForDataUntil: (Socket deadlineSecs: 5)) ifFalse: [
			Transcript show: 'data was late'; cr].
		bytesRead _ ioSocketHandle receiveDataInto: buf 
			startingAt: 1 count: buf size.
		1 to: bytesRead do: [:ii | response nextPut: (buf at: ii)].
			"Any way to do this so we do not have to recopy?"
		].
	response reset.	"position: 0."
	^ response! !

!FTPSocket methodsFor: 'as yet unclassified' stamp: 'dtl 11/14/1999 15:56'!
getAllDataWhileWatching: otherSocket
	"Reel in all data until the server closes the connection.  At the same time, watch for errors on otherSocket.  Return a RWBinaryOrTextStream.  Don't know how much is coming."

	| buf response bytesRead |
	buf _ String new: 4000.
	response _ RWBinaryOrTextStream on: (String new: 4000).
	[self isConnected | self dataAvailable] whileTrue: [
		(self waitForDataUntil: (Socket deadlineSecs: 5)) ifFalse: [
			otherSocket responseError ifTrue: [self destroy. ^ #error:].
			Transcript show: 'data was late'; cr].
		bytesRead _ ioSocketHandle receiveDataInto: buf 
			startingAt: 1 count: buf size.
		1 to: bytesRead do: [:ii | response nextPut: (buf at: ii)].
			"Any way to do this so we do not have to recopy?"
		].
	response reset.	"position: 0."
	^ response! !

!FTPSocket methodsFor: 'as yet unclassified' stamp: 'dtl 11/14/1999 15:56'!
getDataTo: dataStream whileWatching: otherSocket
	"Reel in all data until the server closes the connection.  At the same time, watch for errors on otherSocket.  Don't know how much is coming.  Put the data on the stream."

	| buf bytesRead |
	buf _ String new: 4000.
	[self isConnected | self dataAvailable] whileTrue: [
		(self waitForDataUntil: (Socket deadlineSecs: 5)) ifFalse: [
			otherSocket responseError ifTrue: [self destroy. ^ #error:].
			Transcript show: 'data was late'; cr].
		bytesRead _ ioSocketHandle receiveDataInto: buf 
			startingAt: 1 count: buf size.
		1 to: bytesRead do: [:ii | dataStream nextPut: (buf at: ii)].
			"Any way to do this so we do not have to recopy?"
		].
	dataStream reset.	"position: 0."
	^ dataStream! !


!StandardFileStream methodsFor: 'open/close' stamp: 'dtl 11/13/1999 10:21'!
close
	"Close this file."

	ioHandle ifNotNil: [
		ioHandle close.
		self unregister.
		ioHandle _ nil].
! !

!StandardFileStream methodsFor: 'open/close' stamp: 'dtl 11/13/1999 11:22'!
closed
	"Answer true if this file is closed."

	^ ioHandle isNil or: [ioHandle isOpen not]

! !

!StandardFileStream methodsFor: 'open/close' stamp: 'dtl 11/13/1999 11:20'!
ensureOpen
	"Make sure that this file really is open."

	self closed ifTrue: [^ self reopen].
	ioHandle isOpen ifFalse: [^ self reopen].
	^ self

! !

!StandardFileStream methodsFor: 'open/close' stamp: 'dtl 11/13/1999 10:55'!
open: fileName forWrite: writeMode 
	"Open the file with the given name. If writeMode is true, allow writing, otherwise
	open the file in read-only mode."

	ioHandle _ IOHandle forFile: fileName forWrite: writeMode.
	ioHandle ifNil: [^ nil].  "allows sender to detect failure"
	self register.
	name _ fileName.
	rwmode _ writeMode.
	buffer1 _ String new: 1.
! !

!StandardFileStream methodsFor: 'open/close' stamp: 'dtl 11/13/1999 11:01'!
reopen
	"Close and reopen this file. The file position is reset to zero."
	"Details: Files that were open when a snapshot occurs are no longer valid when
	the snapshot is resumed. This operation re-opens the file if that has happened."

	ioHandle ifNotNil: [
		ioHandle closeNoError.
		ioHandle _ nil].
	self open: name forWrite: rwmode.
! !

!StandardFileStream methodsFor: 'access' stamp: 'dtl 10/23/1999 11:21'!
ioHandle

	^ ioHandle! !

!StandardFileStream methodsFor: 'access' stamp: 'dtl 10/23/1999 11:21'!
ioHandle: anIOHandle

	ioHandle _ anIOHandle! !

!StandardFileStream methodsFor: 'access' stamp: 'dtl 11/13/1999 12:36'!
size
	"Answer the size of the file in characters.  2/12/96 sw"

	^ ioHandle size
! !

!StandardFileStream methodsFor: 'read, write, position' stamp: 'dtl 11/13/1999 11:27'!
atEnd
	"Answer whether the receiver is at its end."

	^ ioHandle atEnd
! !

!StandardFileStream methodsFor: 'read, write, position' stamp: 'dtl 11/13/1999 11:35'!
flush

	ioHandle flush! !

!StandardFileStream methodsFor: 'read, write, position' stamp: 'dtl 11/13/1999 11:42'!
next
	"Answer the next byte from this file, or nil if at the end of the file."

	| count |
	count _ ioHandle readInto: buffer1 startingAt: 1 count: 1.
	count = 1
		ifTrue: [^ buffer1 at: 1]
		ifFalse: [^ nil].
! !

!StandardFileStream methodsFor: 'read, write, position' stamp: 'dtl 11/13/1999 11:53'!
next: n into: aString
	"Read n bytes into the given string. Return the number of bytes read."

	^ ioHandle readInto: aString startingAt: 1 count: n! !

!StandardFileStream methodsFor: 'read, write, position' stamp: 'dtl 11/13/1999 11:51'!
nextInto: aString
	"Fill aString, whose size dictates the size of the read, with characters from the receiver.  1/31/96 sw"

	| count wanted |
	count _ ioHandle readInto: aString startingAt: 1 count: (wanted _ aString size).
	count < wanted ifTrue: [^ aString copyFrom: 1 to: count].
	^ aString! !

!StandardFileStream methodsFor: 'read, write, position' stamp: 'dtl 11/13/1999 12:15'!
nextPut: char
	"Write the given character to this file."

	buffer1 at: 1 put: char.
	ioHandle writeFrom: buffer1 startingAt: 1 count: 1.
	^ char
! !

!StandardFileStream methodsFor: 'read, write, position' stamp: 'dtl 11/13/1999 12:16'!
nextPutAll: aString
	"Write all the characters of the given string to this file."

	ioHandle writeFrom: aString startingAt: 1 count: aString size.
	^ aString
! !

!StandardFileStream methodsFor: 'read, write, position' stamp: 'dtl 11/13/1999 11:29'!
position
	"Return the receiver's current file position.  2/12/96 sw"

	^ ioHandle position
! !

!StandardFileStream methodsFor: 'read, write, position' stamp: 'dtl 11/13/1999 11:31'!
position: pos
	"Set the receiver's position as indicated.  2/12/96 sw"

	^ ioHandle position: pos
! !

!StandardFileStream methodsFor: 'read, write, position' stamp: 'dtl 11/13/1999 12:25'!
readInto: byteArray startingAt: startIndex count: count
	"Read into the given array as specified, and return the count
	actually transferred.  index and count are in units of bytes or
	longs depending on whether the array is Bitmap, String or ByteArray"

	^ ioHandle readInto: byteArray startingAt: startIndex count: count
! !

!StandardFileStream methodsFor: 'read, write, position' stamp: 'dtl 10/23/1999 11:23'!
readOnlyCopy

	^ self class readOnlyFileNamed: self name! !

!StandardFileStream methodsFor: 'finalization' stamp: 'dtl 11/13/1999 12:28'!
actAsExecutor

	super actAsExecutor.
	name := nil.! !

!StandardFileStream methodsFor: 'finalization' stamp: 'dtl 11/13/1999 12:27'!
finalize

	ioHandle closeNoError.
	^ self! !


!StandardFileStream class reorganize!
('file creation' fileNamed: isAFileNamed: newFileNamed: oldFileNamed: readOnlyFileNamed:)
('registry' register: registry unregister:)
!


IOHandle removeSelector: #isBigEndian!
IOHandle removeSelector: #bigEndianMachine!
IOHandle removeSelector: #lastOp:!
IOHandle removeSelector: #oprintOn:!
IOHandle removeSelector: #writable!
IOHandle removeSelector: #forFile:forWrite:!
IOHandle removeSelector: #lastOp!
IOHandle removeSelector: #file:!
IOHandle removeSelector: #isLittleEndian!
IOHandle removeSelector: #printOn:!
IOHandle removeSelector: #writable:!
IOHandle removeSelector: #file!
IOHandle removeSelector: #fileSize:!
IOHandle removeSelector: #forFiles!
IOHandle removeSelector: #forSockets!
IOHandle removeSelector: #onFile:forWrite:!
IOHandle removeSelector: #littleEndianMachine!
IOHandle removeSelector: #fileSize!
IOFileHandle removeSelector: #readNextInto:startingAt:!
IOFileHandle removeSelector: #fileSizeFromSqFileStructUsingAccessor:!
IOFileHandle removeSelector: #lastOpFromSqFileStructUsingAccessor:!
IOFileHandle removeSelector: #changed!
IOFileHandle removeSelector: #sessionIDfromSqFileStruct!
IOFileHandle removeSelector: #writableFromSqFileStructUsingAccessor:!
IOHandle initialize!
IOHandle class removeSelector: #structureSize!
IOHandle class removeSelector: #newFrom:!
IOFileHandle class removeSelector: #sqFileStructureSize!
IOFileHandle class removeSelector: #newFrom:!
IOSocketHandle removeSelector: #primIOSocketHandleRemotePort:!
IOSocketHandle removeSelector: #primIOSocketHandle:listenOn:backlogSize:!
IOSocketHandle removeSelector: #primIOSocketHandleError:!
IOSocketHandle removeSelector: #primAcceptFrom:receiveBufferSize:sendBufSize:semaIndex:!
IOSocketHandle removeSelector: #primIOSocketHandle:receiveDataInto:startingAt:count:!
IOSocketHandle removeSelector: #primIOSocketHandleRemoteAddress:!
IOSocketHandle removeSelector: #primIOSocketHandleAbortConnection:!
IOSocketHandle removeSelector: #primIOSocketHandleDestroyGently:!
IOSocketHandle removeSelector: #acceptFrom:receiveBufferSize:sendBufSize:semaIndex:!
IOSocketHandle removeSelector: #primIOSocketHandle:connectTo:port:!
IOSocketHandle removeSelector: #socketTypeFromInteger:!
IOSocketHandle removeSelector: #socketTypeAsInteger!
IOSocketHandle removeSelector: #primIOSocketHandle:sendData:startIndex:count:!
IOSocketHandle removeSelector: #primIOSocketHandleCloseConnection:!
IOSocketHandle removeSelector: #primIOSocketHandleDestroy:!
IOSocketHandle removeSelector: #primIOSocketHandle:listenOn:!
IOSocketHandle removeSelector: #primIOSocketHandleSendDone:!
IOSocketHandle removeSelector: #primSocketCreateNetwork:type:receiveBufferSize:sendBufSize:semaIndex:!
IOSocketHandle removeSelector: #primIOSocketHandleConnectionStatus:!
IOSocketHandle removeSelector: #changed!
IOSocketHandle removeSelector: #initialize!
IOSocketHandle removeSelector: #primIOSocketHandleReceiveDataAvailable:!
IOSocketHandle removeSelector: #primIOSocketHandle:setPort:!
IOSocketHandle removeSelector: #primIOSocketHandleLocalPort:!
IOSocketHandle removeSelector: #primIOSocketHandleCreateNetwork:type:receiveBufferSize:sendBufSize:semaIndex:!
IOSocketHandle removeSelector: #socketTypeFromSqSocketStructUsingAccessor:!
IOSocketHandle removeSelector: #primIOSocketHandleLocalAddress:!
IOSocketHandle initialize!
IOSocketHandle class removeSelector: #sqSocketStructureSize!
Socket removeSelector: #primSocketError:!
Socket removeSelector: #primSocket:setPort:!
Socket removeSelector: #primSocketSendDone:!
Socket removeSelector: #primSocketRemoteAddress:!
Socket removeSelector: #primSocketAbortConnection:!
Socket removeSelector: #primAcceptFrom:receiveBufferSize:sendBufSize:semaIndex:!
Socket removeSelector: #primSocketCreateNetwork:type:receiveBufferSize:sendBufSize:semaIndex:!
Socket removeSelector: #primSocket:connectTo:port:!
Socket removeSelector: #primSocket:listenOn:!
Socket removeSelector: #socketHandle!
Socket removeSelector: #primSocketConnectionStatus:!
Socket removeSelector: #primSocketLocalAddress:!
Socket removeSelector: #primSocketCloseConnection:!
Socket removeSelector: #primSocketRemotePort:!
Socket removeSelector: #primSocketReceiveDataAvailable:!
Socket removeSelector: #primSocket:sendData:startIndex:count:!
Socket removeSelector: #primSocket:receiveDataInto:startingAt:count:!
Socket removeSelector: #primSocketDestroyGently:!
Socket removeSelector: #primSocket:listenOn:backlogSize:!
Socket removeSelector: #primSocketDestroy:!
Socket removeSelector: #primSocketLocalPort:!
StandardFileStream removeSelector: #primCloseNoError:!
StandardFileStream removeSelector: #primOpen:writable:!
StandardFileStream removeSelector: #fileID!
StandardFileStream removeSelector: #primSize:!
StandardFileStream removeSelector: #primWrite:from:startingAt:count:!
StandardFileStream removeSelector: #name:attachTo:!
StandardFileStream removeSelector: #primAtEnd:!
StandardFileStream removeSelector: #primClose:!
StandardFileStream removeSelector: #primRead:into:startingAt:count:!
StandardFileStream removeSelector: #primSizeNoError:!
StandardFileStream removeSelector: #primGetPosition:!
StandardFileStream removeSelector: #primSetPosition:to:!
"Postscript:

Restore the copy of the original FileStream>>concreteStream method
by copying it back from the ATemporaryStandardFileStream class."
FileStream class removeSelector: #concreteStream.
FileStream class
	copy: #concreteStream
	from: ATemporaryMethodSaver class
	classified: 'concrete classes'.

"Reopen the sources and changes files using the updated classes."
(SourceFiles at: 1) close.
SourceFiles at: 1 put: (FileStream oldFileNamed: (SourceFiles at: 1) name).
(SourceFiles at: 2) close.
SourceFiles at: 2 put: (FileStream oldFileNamed: (SourceFiles at: 2) name).

"Clean up the temporary classes."
Smalltalk removeClassFromSystem: ATemporaryMethodSaver.
Smalltalk removeClassFromSystem: TempHtmlFileStream.
Smalltalk removeClassFromSystem: TempCrLfFileStream.
Smalltalk removeClassFromSystem: TempStandardFileStream.
!

